Package: dsTidyverseClient
Type: Package
Title: 'DataSHIELD' 'Tidyverse' Clientside Package
Version: 1.1.0
Authors@R: 
    c(
      person(given = "Tim",
             family = "Cadman",
             role = c("aut", "cre"),
             email = "t.j.cadman@umcg.nl",
             comment = c(ORCID = "0000-0002-7682-5645")), 
     person(given = "Mariska",
             family = "Slofstra",
             role = c("aut"),
             email = "m.k.slofstra@umcg.nl",
             comment = c(ORCID = "0000-0002-0400-0468")),
     person(given = "Stuart",
            family = "Wheater",
            role = c("aut")),
     person(given = "Demetris",
            family = "Avraam",
            role = c("aut"))
       )
Maintainer: Tim Cadman <t.j.cadman@umcg.nl>
Description: Implementation of selected 'Tidyverse' functions within 'DataSHIELD', an open-source federated analysis solution in R. Currently, 'DataSHIELD' contains very limited tools for data manipulation, so the aim of this package is to improve the researcher experience by implementing essential functions for data manipulation, including subsetting, filtering, grouping, and renaming variables. This is the client-side package which should be installed locally, and is used in conjunction with the server-side package 'dsTidyverse' which is installed on the remote server holding the data. For more information, see <https://tidyverse.org/> and <https://datashield.org/>.
License: LGPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: DSI (>= 1.7.0), cli, rlang, assertthat
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, DSLite, dsBase,
        dsBaseClient, dsTidyverse, dplyr
Config/testthat/edition: 3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-29 07:19:31 UTC; tcadman
Author: Tim Cadman [aut, cre] (ORCID: <https://orcid.org/0000-0002-7682-5645>),
  Mariska Slofstra [aut] (ORCID: <https://orcid.org/0000-0002-0400-0468>),
  Stuart Wheater [aut],
  Demetris Avraam [aut]
Repository: CRAN
Date/Publication: 2026-04-29 07:30:13 UTC
Built: R 4.6.0; ; 2026-04-29 11:11:43 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ds.arrange)
export(ds.as_tibble)
export(ds.bind_cols)
export(ds.bind_rows)
export(ds.case_when)
export(ds.distinct)
export(ds.filter)
export(ds.group_by)
export(ds.group_keys)
export(ds.if_else)
export(ds.mutate)
export(ds.rename)
export(ds.select)
export(ds.slice)
export(ds.ungroup)
importFrom(DSI,datashield.aggregate)
importFrom(DSI,datashield.assign)
importFrom(DSI,datashield.connections_find)
importFrom(assertthat,assert_that)
importFrom(cli,cli_abort)
importFrom(rlang,enquo)
importFrom(rlang,quo_text)
importFrom(rlang,sym)

# dsTidyverseClient
<!-- badges: start -->
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/dsTidyverseClient)](https://CRAN.R-project.org/package=dsTidyverseClient)
[![CircleCI](https://circleci.com/gh/molgenis/ds-tidyverse-client.svg?style=shield)](https://app.circleci.com/pipelines/github/molgenis/ds-tidyverse-client)
[![codecov](https://codecov.io/gh/molgenis/ds-tidyverse-client/branch/master/graph/badge.svg?token=ITPMERAWYI)](https://app.codecov.io/gh/molgenis/ds-tidyverse-client)
<!-- badges: end -->

This is the DataSHIELD implementation of selected [Tidyverse](https://tidyverse.org/) functions. 
Currently most of these functions are from dplyr, however in the future functions from other
Tidyverse packages can be included.

## Installation
- Install dsTidyverse on your armadillo or opal server.
- Whitelist dsTidyverse if required
- Install dsTidyverseClient on your local machine

## Functions/Features
### From the `dplyr` package:
- **`select`**: Choose columns from a data frame.
- **`rename`**: Rename columns in a data frame.
- **`mutate`**: Create or modify columns.
- **`if_else`**: A vectorized conditional function.
- **`case_when`**: A general vectorized conditional function.
- **`bind_cols`**: Combine data frames by columns.
- **`bind_rows`**: Combine data frames by rows.
- **`filter`**: Filter rows based on conditions.
- **`slice`**: Select rows by position.
- **`arrange`**: Arrange rows by values of a column or multiple columns.
- **`group_by`**: Group data by one or more columns.
- **`ungroup`**: Remove grouping from data.
- **`group_keys`**: Retrieve the group keys from a grouped data frame.
- **`distinct`**: Return unique rows based on certain columns.

### From the `tibble` package:
- **`as_tibble`**: Convert data to a tibble.

## Privacy control 
For details on options to manage privacy levels within dsTidyverse, see [vignette](
https://molgenis.github.io/ds-tidyverse/articles/tidyversePrivacy.html).

## Contributing
If there are functions in this list you would like implemented, please either attempt to do so 
yourself and submit a pull request, or submit a feature request in the `issues` section. See 
the "CONTRIBUTING" file for more information.

#' @title Vectorised if-else
#' @description DataSHIELD implementation of \code{dplyr::if_else}.
#' @param condition A list specifying a logical vector in tidyverse syntax, ie data and column names unquoted.
#' @param true Vector to use for TRUE value of condition.
#' @param false Vector to use for FALSE value of condition.
#' @param missing If not NULL, will be used as the value for NA values of condition. Follows the same size and type rules as true and false.
#' @param ptype An optional prototype declaring the desired output type. If supplied, this overrides the common type of true, false, and missing.
#' @param size An optional size declaring the desired output size. If supplied, this overrides the size of condition.
#' @param newobj Character specifying name for new server-side data frame.
#' @param datasources datashield connections object.
#' @return No return value, called for its side effects. A vector with the same size as
#' \code{condition} and the same type as the common type of \code{true}, \code{false}, and
#' \code{missing} is created on the server.
#' @importFrom DSI datashield.assign
#' @importFrom rlang enquo
#' @examples
#' \dontrun{
#' ## First log in to a DataSHIELD session with mtcars dataset loaded.
#'
# ` ds.if_else(
# `  condition = list(mtcars$mpg > 20),
# `. true = "high",
# `. false = "low",
# `. newobj = "new_var")
#'
#' ## Refer to the package vignette for more examples.
#' }
#' @export
ds.if_else <- function(condition = NULL, true = NULL, false = NULL, missing = NULL,
                       ptype = NULL, size = NULL, newobj = NULL, datasources = NULL) {
  tidy_select <- .format_args_as_string(enquo(condition))
  datasources <- .set_datasources(datasources)
  .check_tidy_args(NULL, newobj, check_df = FALSE)
  cally <- .make_serverside_call("ifElseDS", tidy_select, list(true, false, missing, ptype, size))
  datashield.assign(datasources, newobj, cally)
}

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

mtcars <- mtcars %>% mutate(cat_var = factor(ifelse(mpg > 20, "high", "low")))
login_data <- .prepare_dslite(assign_method = "ifElseDS", tables = list(mtcars = mtcars))
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")

test_that("ds.if_else correctly passes argument with numeric condition and categorical outcome", {
  skip_if_not_installed("dsBaseClient")
  ds.if_else(
    condition = list(mtcars$mpg > 20),
    "high",
    "low",
    newobj = "test",
    datasources = conns
  )

  nqmes <- names(ds.table("test", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts)

  expect_equal(
    nqmes,
    c("high", "low", "NA")
  )

  expect_equal(
    ds.class("test", datasources = conns)[[1]],
    "character"
  )

  counts <- ds.table("test", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts
  expect_equal(
    as.numeric(counts),
    c(42, 54, 0)
  )
})

test_that("ds.if_else correctly passes argument with numeric condition and numerical outcome", {
  skip_if_not_installed("dsBaseClient")
  ds.if_else(
    condition = list(mtcars$mpg > 20),
    99,
    100,
    newobj = "test",
    datasources = conns
  )

  nqmes <- names(ds.table("test", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts)

  expect_equal(
    nqmes,
    c("99", "100", "NA")
  )

  expect_equal(
    ds.class("test", datasources = conns)[[1]],
    "numeric"
  )

  counts <- ds.table("test", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts
  expect_equal(
    as.numeric(counts),
    c(42, 54, 0)
  )
})

test_that("ds.if_else correctly passes argument with = ", {
  skip_if_not_installed("dsBaseClient")
  ds.if_else(
    condition = list(mtcars$vs == "0"),
    "no",
    "yes",
    newobj = "testcat",
    datasources = conns
  )

  names <- names(ds.table("testcat", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts)

  expect_equal(
    names,
    c("no", "yes", "NA")
  )

  expect_equal(
    ds.class("testcat", datasources = conns)[[1]],
    "character"
  )

  counts <- ds.table("testcat", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts
  expect_equal(
    as.numeric(counts),
    c(54, 42, 0)
  )
})

#' Create a DSLite login object that can be used for testing
#'
#' @param assign_method A string specifying the name of the custom assign method to be added
#'   to the DSLite server. If `NULL`, no additional assign method is added. Default is `NULL`.
#' @param aggregate_method A string specifying the name of the custom aggregate method to be
#'   added to the DSLite server. If `NULL`, no additional aggregate method is added. Default is `NULL`.
#' @param tables A named list of tables to be made available on the DSLite server. Default is `NULL`.
#'
#' @return A DataSHIELD login object containing the necessary connection information for the DSLite server.
#'
#' @examples
#' \dontrun{
#' # Prepare a DSLite server with default methods and custom assign/aggregate methods
#' login_data <- .prepare_dslite(
#'   assign_method = "customAssign",
#'   aggregate_method = "customAggregate",
#'   tables = list(mtcars = mtcars, mtcars_group = mtcars_group)
#' )
#' }
#' @importFrom DSLite newDSLiteServer
#' @importFrom DSI newDSLoginBuilder
#' @export
.prepare_dslite <- function(assign_method = NULL, aggregate_method = NULL, tables = NULL) {
  options(datashield.env = environment())
  dslite.server <- DSLite::newDSLiteServer(tables = tables)
  dslite.server$config(defaultDSConfiguration(include = c("dsBase", "dsTidyverse")))
  dslite.server$aggregateMethod("exists", "base::exists")
  dslite.server$aggregateMethod("classDS", "dsBase::classDS")
  dslite.server$aggregateMethod("lsDS", "dsBase::lsDS")
  dslite.server$aggregateMethod("dsListDisclosureSettings", "dsTidyverse::dsListDisclosureSettings")

  if (!is.null(assign_method)) {
    dslite.server$assignMethod(assign_method, paste0("dsTidyverse::", assign_method))
  }

  if (!is.null(aggregate_method)) {
    dslite.server$aggregateMethod(assign_method, paste0("dsTidyverse::", assign_method))
  }

  builder <- DSI::newDSLoginBuilder()
  builder$append(server = "server_1", url = "dslite.server", driver = "DSLiteDriver")
  builder$append(server = "server_2", url = "dslite.server", driver = "DSLiteDriver")
  builder$append(server = "server_3", url = "dslite.server", driver = "DSLiteDriver")
  login_data <- builder$build()
  return(login_data)
}

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

data("mtcars")
login_data <- .prepare_dslite(assign_method = "selectDS", tables = list(mtcars = mtcars))
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")

test_that("ds.select fails with correct error message if data not present ", {
  skip_if_not_installed("dsBaseClient")
  expect_error(
    ds.select(
      df.name = "datanotthere",
      tidy_expr = list(mpg:drat),
      newobj = "nodata",
      datasources = conns
    )
  )
})

test_that("ds.select correctly passes : ", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(mpg:drat),
    newobj = "mpg_drat",
    datasources = conns
  )

  expect_equal(
    ds.colnames("mpg_drat", datasources = conns)[[1]],
    c("mpg", "cyl", "disp", "hp", "drat")
  )
})

test_that("ds.select correctly passes `starts_with`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(starts_with("m")),
    newobj = "starts",
    datasources = conns
  )

  expect_equal(
    ds.colnames("starts", datasources = conns)[[1]],
    "mpg"
  )
})

test_that("ds.select correctly passes `ends_with`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(ends_with("m")),
    newobj = "ends",
    datasources = conns
  )

  expect_equal(
    ds.colnames("ends", datasources = conns)[[1]],
    "am"
  )
})

test_that("ds.select correctly passes `matches`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(matches("a")),
    newobj = "matches",
    datasources = conns
  )

  expect_equal(
    ds.colnames("matches", datasources = conns)[[1]],
    c("drat", "am", "gear", "carb")
  )
})

test_that("ds.select correctly passes `everything`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(everything()),
    newobj = "everything",
    datasources = conns
  )

  expect_equal(
    ds.colnames("everything", datasources = conns)[[1]],
    colnames(mtcars)
  )
})

test_that("ds.select correctly passes `last_col`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(last_col()),
    newobj = "last",
    datasources = conns
  )

  expect_equal(
    ds.colnames("last", datasources = conns)[[1]],
    "carb"
  )
})

test_that("ds.select correctly passes `group_cols`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(group_cols()),
    newobj = "group",
    datasources = conns
  )

  expect_equal(
    ds.colnames("group", datasources = conns)[[1]],
    character(0)
  )
})

test_that("ds.select correctly passes strings with '&'", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(starts_with("c") & ends_with("b")),
    newobj = "and",
    datasources = conns
  )

  expect_equal(
    ds.colnames("and", datasources = conns)[[1]],
    "carb"
  )
})

test_that("ds.select correctly passes strings with '!'", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(!mpg),
    newobj = "not",
    datasources = conns
  )

  expect_equal(
    ds.colnames("not", datasources = conns)[[1]],
    c("cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb")
  )
})

test_that("ds.select correctly passes strings with '|'", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(starts_with("c") | ends_with("b")),
    newobj = "or",
    datasources = conns
  )

  expect_equal(
    ds.colnames("or", datasources = conns)[[1]],
    c("cyl", "carb")
  )
})

test_that("ds.select correctly passes `strings with `all_of`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(all_of(c("mpg", "cyl"))),
    newobj = "all_of",
    datasources = conns
  )

  expect_equal(
    ds.colnames("all_of", datasources = conns)[[1]],
    c("mpg", "cyl")
  )
})

test_that("ds.select correctly passes strings with `any_of`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(any_of(c("mpg", "cyl"))),
    newobj = "any_of",
    datasources = conns
  )

  expect_equal(
    ds.colnames("any_of", datasources = conns)[[1]],
    c("mpg", "cyl")
  )
})

test_that("ds.select correctly passes complex strings", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list((starts_with("c") & ends_with("b")) | contains("ra") | gear:carb),
    newobj = "complex",
    datasources = conns
  )

  expect_equal(
    ds.colnames("complex", datasources = conns)[[1]],
    c("carb", "drat", "gear")
  )
})

test_that("ds.select correctly passes strings with `where`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(where(is.numeric)),
    newobj = "where",
    datasources = conns
  )

  expect_equal(
    ds.colnames("where", datasources = conns)[[1]],
    colnames(mtcars)
  )
})

test_that("ds.select correctly passes strings with '='", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(test = mpg, cyl, gear),
    newobj = "equals",
    datasources = conns
  )

  expect_equal(
    ds.colnames("equals", datasources = conns)[[1]],
    c("test", "cyl", "gear")
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

login_data <- .prepare_dslite(assign_method = "distinctDS", tables = list(mtcars = mtcars))
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")

test_that("ds.distinct correctly identified distinct rows", {
  skip_if_not_installed("dsBaseClient")
  ds.distinct(
    df.name = "mtcars",
    tidy_expr = list(cyl, carb),
    newobj = "dist_df",
    datasources = conns
  )

  expect_equal(
    ds.class("dist_df", datasources = conns)[[1]],
    "data.frame"
  )

  expect_equal(
    ds.dim("dist_df", datasources = conns)[[1]],
    c(9, 2)
  )
})

test_that("ds.distinct works with where `tidy_expr` arg is empty", {
  skip_if_not_installed("dsBaseClient")
  ds.distinct(
    df.name = "mtcars",
    newobj = "dist_df",
    datasources = conns
  )

  expect_equal(
    ds.dim("dist_df", datasources = conns)[[1]],
    c(32, 11)
  )

  expect_equal(
    ds.colnames("dist_df", datasources = conns)[[1]],
    c("mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb")
  )
})

test_that("ds.distinct works with `.keep_all` argument", {
  skip_if_not_installed("dsBaseClient")
  ds.distinct(
    df.name = "mtcars",
    tidy_expr = list(cyl, carb),
    .keep_all = TRUE,
    newobj = "dist_df",
    datasources = conns
  )

  expect_equal(
    ds.dim("dist_df", datasources = conns)[[1]],
    c(9, 11)
  )

  expect_equal(
    ds.colnames("dist_df", datasources = conns)[[1]],
    c("mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb")
  )

  ds.distinct(
    df.name = "mtcars",
    tidy_expr = list(cyl, carb),
    .keep_all = FALSE,
    newobj = "dist_df",
    datasources = conns
  )

  expect_equal(
    ds.dim("dist_df", datasources = conns)[[1]],
    c(9, 2)
  )

  expect_equal(
    ds.colnames("dist_df", datasources = conns)[[1]],
    c("cyl", "carb")
  )
})

test_that("ds.distinct fails if created subset is too small", {
  skip_if_not_installed("dsBaseClient")
  expect_error(
    ds.distinct(
      df.name = "mtcars",
      tidy_expr = list(vs),
      newobj = "dist_df",
      datasources = conns
    )
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

login_data <- .prepare_dslite(assign_method = "bindRowsDS", tables = list(mtcars = mtcars))
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")

test_that("ds.bind_rows binds two data frames together", {
  skip_if_not_installed("dsBaseClient")
  ds.bind_rows(
    to_combine = list(mtcars, mtcars),
    newobj = "newnew",
    .id = NULL,
    datasources = conns
  )

  expect_equal(
    ds.class("newnew", datasources = conns)[[1]],
    "data.frame"
  )

  expect_equal(
    ds.dim("newnew", datasources = conns)[[1]],
    c(64, 11)
  )

  expect_equal(
    ds.colnames("newnew", datasources = conns)[[1]],
    c("mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb")
  )
})

test_that("ds.case_when works with .id arg", {
  skip_if_not_installed("dsBaseClient")
  ds.bind_rows(
    to_combine = list(mtcars, mtcars),
    .id = "which_df",
    newobj = "test",
    datasources = conns
  )

  expect_equal(
    ds.class("test", datasources = conns)[[1]],
    "data.frame"
  )

  expect_equal(
    ds.dim("test", datasources = conns)[[1]],
    c(64, 12)
  )

  expect_equal(
    ds.colnames("test", datasources = conns)[[1]],
    c("which_df", "mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb")
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

data(mtcars)
login_data <- .prepare_dslite(assign_method = "mutateDS", tables = list(mtcars = mtcars))
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")

test_that("ds.mutate correctly passes good argument", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate(
    df.name = "mtcars",
    tidy_expr = list(mpg_trans = cyl * 1000, new_var = (hp - drat) / qsec),
    newobj = "new",
    datasources = conns
  )
  expected_cols <- c("mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb", "mpg_trans", "new_var")

  expect_equal(
    ds.colnames("new", datasources = conns)[[1]],
    expected_cols
  )

  expected_mean <- ds.mean("new$mpg_trans", datasources = conns)
  expect_equal(
    mean(expected_mean$Mean.by.Study[1]),
    6187.5
  )
})

test_that("ds.mutate fails with bad argument argument", {
  skip_if_not_installed("dsBaseClient")
  expect_error(
    ds.mutate(
      df.name = "mtcars",
      tidy_expr = list(mpg_trans = cyl * 1000, new_var = (hp - drat) / qsec, filterasd("asdasdf")),
      newobj = "new",
      datasources = conns
    )
  )

  expect_error(
    ds.mutate(
      df.name = "mtcars",
      tidy_expr = list(mpg_trans = cyl * 1000, new_var = (hp - drat) / qsec),
      newobj = "new",
      .keep = NULL,
      datasources = conns
    )
  )
})

test_that("ds.mutate passes with different .keep argument", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate(
    df.name = "mtcars",
    tidy_expr = list(mpg_trans = cyl * 1000, new_var = (hp - drat) / qsec),
    newobj = "new",
    .keep = "none",
    .before = NULL,
    .after = NULL,
    datasources = conns
  )

  expect_equal(
    ds.colnames("new", datasources = conns)[[1]],
    c("mpg_trans", "new_var")
  )
})

test_that("ds.mutate passes with different .before argument", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate(
    df.name = "mtcars",
    tidy_expr = list(mpg_trans = cyl * 1000, new_var = (hp - drat) / qsec),
    newobj = "new",
    .keep = "all",
    .before = "disp",
    .after = NULL,
    datasources = conns
  )

  expect_equal(
    ds.colnames("new", datasources = conns)[[1]],
    c("mpg", "cyl", "mpg_trans", "new_var", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb")
  )
})

test_that("ds.mutate passes with different .after argument", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate(
    df.name = "mtcars",
    tidy_expr = list(mpg_trans = cyl * 1000, new_var = (hp - drat) / qsec),
    newobj = "new",
    .keep = "all",
    .before = NULL,
    .after = "qsec",
    datasources = conns
  )

  expect_equal(
    ds.colnames("new", datasources = conns)[[1]],
    c("mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "mpg_trans", "new_var", "vs", "am", "gear", "carb")
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

data("mtcars")
num_range_df <- data.frame(x1 = 1:5, x2 = 6:10, x3 = 11:15, y1 = 16:20, stringsAsFactors = FALSE)
login_data <- .prepare_dslite(assign_method = "selectDS", tables = list(mtcars = mtcars, num_range_df = num_range_df))
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")
datashield.assign.table(conns, "num_range_df", "num_range_df")

test_that("ds.select correctly passes `starts_with`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(starts_with("m")),
    newobj = "starts",
    datasources = conns
  )

  expect_equal(
    ds.colnames("starts", datasources = conns)[[1]],
    "mpg"
  )
})

test_that("ds.select correctly passes `ends_with`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(ends_with("m")),
    newobj = "ends",
    datasources = conns
  )

  expect_equal(
    ds.colnames("ends", datasources = conns)[[1]],
    "am"
  )
})

test_that("ds.select correctly passes `matches`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(matches("a")),
    newobj = "matches",
    datasources = conns
  )

  expect_equal(
    ds.colnames("matches", datasources = conns)[[1]],
    c("drat", "am", "gear", "carb")
  )
})

test_that("ds.select correctly passes `everything`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(everything()),
    newobj = "everything",
    datasources = conns
  )

  expect_equal(
    ds.colnames("everything", datasources = conns)[[1]],
    colnames(mtcars)
  )
})

test_that("ds.select correctly passes `last_col`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(last_col()),
    newobj = "last",
    datasources = conns
  )

  expect_equal(
    ds.colnames("last", datasources = conns)[[1]],
    "carb"
  )
})

test_that("ds.select correctly passes `group_cols`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(group_cols()),
    newobj = "group",
    datasources = conns
  )

  expect_equal(
    ds.colnames("group", datasources = conns)[[1]],
    character(0)
  )
})

test_that("ds.select correctly passes `all_of`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(all_of(c("mpg", "cyl"))),
    newobj = "all_of",
    datasources = conns
  )

  expect_equal(
    ds.colnames("all_of", datasources = conns)[[1]],
    c("mpg", "cyl")
  )
})

test_that("ds.select correctly passes `any_of`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(any_of(c("mpg", "cyl"))),
    newobj = "any_of",
    datasources = conns
  )

  expect_equal(
    ds.colnames("any_of", datasources = conns)[[1]],
    c("mpg", "cyl")
  )
})

test_that("ds.select correctly passes `contains`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(contains("ra")),
    newobj = "contains",
    datasources = conns
  )

  expect_equal(
    ds.colnames("contains", datasources = conns)[[1]],
    "drat"
  )
})

test_that("ds.select correctly passes `where`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "mtcars",
    tidy_expr = list(where(is.numeric)),
    newobj = "where",
    datasources = conns
  )

  expect_equal(
    ds.colnames("where", datasources = conns)[[1]],
    colnames(mtcars)
  )
})

test_that("ds.select correctly passes `num_range`", {
  skip_if_not_installed("dsBaseClient")
  ds.select(
    df.name = "num_range_df",
    tidy_expr = list(num_range("x", 1:2)),
    newobj = "num_range_res",
    datasources = conns
  )

  expect_equal(
    ds.colnames("num_range_res", datasources = conns)[[1]],
    c("x1", "x2")
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)
require(dsBase)

login_data <- .prepare_dslite(assign_method = "arrangeDS", tables = list(mtcars = mtcars))
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")

test_that("ds.arrange doesn't return error with correct arguments", {
  skip_if_not_installed("dsBaseClient")
  print(listDisclosureSettingsDS())
  ds.arrange(
    df.name = "mtcars",
    tidy_expr = list(cyl),
    newobj = "ordered_df",
    datasources = conns
  )

  expect_equal(
    ds.class("ordered_df", datasources = conns)[[1]],
    "data.frame"
  )
})

test_that("ds.arrange doesn't return error with .by_group argument", {
  skip_if_not_installed("dsBaseClient")
  ds.arrange(
    df.name = "mtcars",
    tidy_expr = list(cyl),
    .by_group = TRUE,
    newobj = "ordered_df",
    datasources = conns
  )

  expect_equal(
    ds.class("ordered_df", datasources = conns)[[1]],
    "data.frame"
  )
})

test_that("ds.arrange returns error if data doesn't exist", {
  skip_if_not_installed("dsBaseClient")
  expect_error(
    ds.arrange(
      df.name = "doesnt_exist",
      tidy_expr = list(cyl),
      newobj = "ordered_df",
      datasources = conns
    )
  )
})

test_that("ds.arrange works with desc() specification", {
  skip_if_not_installed("dsBaseClient")
  ds.arrange(
    df.name = "mtcars",
    tidy_expr = list(desc(cyl)),
    newobj = "desc_df",
    datasources = conns
  )

  expect_equal(
    ds.class("desc_df", datasources = conns)[[1]],
    "data.frame"
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

data(mtcars)
na_df <- data.frame(x = c(1, NA, 3, NA, 5), stringsAsFactors = FALSE)
date_df <- data.frame(date_str = c("2024-01-15", "2024-06-30", "2024-12-25"), stringsAsFactors = FALSE)
date_dmy_df <- data.frame(date_str = c("15/01/2024", "30/06/2024", "25/12/2024"), stringsAsFactors = FALSE)
date_mdy_df <- data.frame(date_str = c("01/15/2024", "06/30/2024", "12/25/2024"), stringsAsFactors = FALSE)
date_dmy_dash_df <- data.frame(date_str = c("15-01-2024", "30-06-2024", "25-12-2024"), stringsAsFactors = FALSE)
date_ymd_slash_df <- data.frame(date_str = c("2024/01/15", "2024/06/30", "2024/12/25"), stringsAsFactors = FALSE)
date_dby_df <- data.frame(date_str = c("15 Jan 2024", "30 Jun 2024", "25 Dec 2024"), stringsAsFactors = FALSE)
date_bdy_df <- data.frame(date_str = c("Jan 15, 2024", "Jun 30, 2024", "Dec 25, 2024"), stringsAsFactors = FALSE)
date_compact_df <- data.frame(date_str = c("20240115", "20240630", "20241225"), stringsAsFactors = FALSE)
date_dot_df <- data.frame(date_str = c("15.01.2024", "30.06.2024", "25.12.2024"), stringsAsFactors = FALSE)
login_data <- .prepare_dslite(
  assign_method = "mutateDS",
  tables = list(
    mtcars = mtcars, na_df = na_df, date_df = date_df,
    date_dmy_df = date_dmy_df, date_mdy_df = date_mdy_df,
    date_dmy_dash_df = date_dmy_dash_df, date_ymd_slash_df = date_ymd_slash_df,
    date_dby_df = date_dby_df, date_bdy_df = date_bdy_df,
    date_compact_df = date_compact_df, date_dot_df = date_dot_df
  )
)
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")
datashield.assign.table(conns, "na_df", "na_df")
datashield.assign.table(conns, "date_df", "date_df")
datashield.assign.table(conns, "date_dmy_df", "date_dmy_df")
datashield.assign.table(conns, "date_mdy_df", "date_mdy_df")
datashield.assign.table(conns, "date_dmy_dash_df", "date_dmy_dash_df")
datashield.assign.table(conns, "date_ymd_slash_df", "date_ymd_slash_df")
datashield.assign.table(conns, "date_dby_df", "date_dby_df")
datashield.assign.table(conns, "date_bdy_df", "date_bdy_df")
datashield.assign.table(conns, "date_compact_df", "date_compact_df")
datashield.assign.table(conns, "date_dot_df", "date_dot_df")

# Math functions

test_that("ds.mutate permits exp()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = exp(cyl)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(exp(mtcars$cyl)), 2)
  )
})

test_that("ds.mutate permits sqrt()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = sqrt(hp)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(sqrt(mtcars$hp)), 2)
  )
})

test_that("ds.mutate permits round()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = round(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1],
    mean(round(mtcars$mpg))
  )
})

test_that("ds.mutate permits floor()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = floor(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1],
    mean(floor(mtcars$mpg))
  )
})

test_that("ds.mutate permits ceiling()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = ceiling(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1],
    mean(ceiling(mtcars$mpg))
  )
})

test_that("ds.mutate permits abs()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = abs(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(abs(mtcars$mpg)), 2)
  )
})

# Stats functions

test_that("ds.mutate permits mean()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = mean(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(mtcars$mpg), 2)
  )
})

test_that("ds.mutate permits median()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = median(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1],
    median(mtcars$mpg)
  )
})

test_that("ds.mutate permits sd()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = sd(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(sd(mtcars$mpg), 2)
  )
})

test_that("ds.mutate permits var()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = var(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(var(mtcars$mpg), 2)
  )
})

# Trig functions

test_that("ds.mutate permits sin()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = sin(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(sin(mtcars$mpg)), 2)
  )
})

test_that("ds.mutate permits cos()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = cos(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(cos(mtcars$mpg)), 2)
  )
})

test_that("ds.mutate permits tan()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = tan(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(tan(mtcars$mpg)), 2)
  )
})

test_that("ds.mutate permits asin()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = asin(mpg / 34)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(asin(mtcars$mpg / 34)), 2)
  )
})

test_that("ds.mutate permits acos()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = acos(mpg / 34)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(acos(mtcars$mpg / 34)), 2)
  )
})

test_that("ds.mutate permits atan()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = atan(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(atan(mtcars$mpg)), 2)
  )
})

# Type conversion functions

test_that("ds.mutate permits as.character()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = as.character(cyl)), newobj = "res", datasources = conns)
  expect_equal(
    ds.class("res$new_col", datasources = conns)[[1]],
    "character"
  )
})

test_that("ds.mutate permits as.integer()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = as.integer(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    ds.class("res$new_col", datasources = conns)[[1]],
    "integer"
  )
  expect_equal(
    ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1],
    mean(as.integer(mtcars$mpg))
  )
})

test_that("ds.mutate permits as.numeric()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = as.numeric(cyl)), newobj = "res", datasources = conns)
  expect_equal(
    ds.class("res$new_col", datasources = conns)[[1]],
    "numeric"
  )
  expect_equal(
    ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1],
    mean(as.numeric(mtcars$cyl))
  )
})

test_that("ds.mutate permits as.Date() with ISO format", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("date_df", tidy_expr = list(new_col = as.Date(date_str)), newobj = "res", datasources = conns)
  expect_equal(
    ds.class("res$new_col", datasources = conns)[[1]],
    "Date"
  )
})

test_that("ds.mutate permits as.Date() with positional format argument", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("date_dmy_df", tidy_expr = list(date_num = as.numeric(as.Date(date_str, "%d/%m/%Y"))), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$date_num", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(as.numeric(as.Date(c("15/01/2024", "30/06/2024", "25/12/2024"), "%d/%m/%Y"))), 2)
  )
})

test_that("ds.mutate as.Date() without format gives wrong result for non-ISO dates", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("date_dmy_df", tidy_expr = list(date_num = as.numeric(as.Date(date_str))), newobj = "res_wrong", datasources = conns)
  correct_mean <- round(mean(as.numeric(as.Date(c("15/01/2024", "30/06/2024", "25/12/2024"), "%d/%m/%Y"))), 2)
  wrong_mean <- round(ds.mean("res_wrong$date_num", datasources = conns)$Mean.by.Study[1, 1], 2)
  expect_false(wrong_mean == correct_mean)
})

test_that("ds.mutate permits as.Date() with US format %m/%d/%Y", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("date_mdy_df", tidy_expr = list(date_num = as.numeric(as.Date(date_str, "%m/%d/%Y"))), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$date_num", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(as.numeric(as.Date(c("01/15/2024", "06/30/2024", "12/25/2024"), "%m/%d/%Y"))), 2)
  )
})

test_that("ds.mutate permits as.Date() with dash format %d-%m-%Y", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("date_dmy_dash_df", tidy_expr = list(date_num = as.numeric(as.Date(date_str, "%d-%m-%Y"))), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$date_num", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(as.numeric(as.Date(c("15-01-2024", "30-06-2024", "25-12-2024"), "%d-%m-%Y"))), 2)
  )
})

test_that("ds.mutate permits as.Date() with %Y/%m/%d format", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("date_ymd_slash_df", tidy_expr = list(date_num = as.numeric(as.Date(date_str, "%Y/%m/%d"))), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$date_num", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(as.numeric(as.Date(c("2024/01/15", "2024/06/30", "2024/12/25"), "%Y/%m/%d"))), 2)
  )
})

test_that("ds.mutate permits as.Date() with abbreviated month %d %b %Y", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("date_dby_df", tidy_expr = list(date_num = as.numeric(as.Date(date_str, "%d %b %Y"))), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$date_num", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(as.numeric(as.Date(c("15 Jan 2024", "30 Jun 2024", "25 Dec 2024"), "%d %b %Y"))), 2)
  )
})

test_that("ds.mutate permits as.Date() with %b %d, %Y format", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("date_bdy_df", tidy_expr = list(date_num = as.numeric(as.Date(date_str, "%b %d, %Y"))), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$date_num", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(as.numeric(as.Date(c("Jan 15, 2024", "Jun 30, 2024", "Dec 25, 2024"), "%b %d, %Y"))), 2)
  )
})

test_that("ds.mutate permits as.Date() with compact %Y%m%d format", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("date_compact_df", tidy_expr = list(date_num = as.numeric(as.Date(date_str, "%Y%m%d"))), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$date_num", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(as.numeric(as.Date(c("20240115", "20240630", "20241225"), "%Y%m%d"))), 2)
  )
})

test_that("ds.mutate permits as.Date() with dot format %d.%m.%Y", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("date_dot_df", tidy_expr = list(date_num = as.numeric(as.Date(date_str, "%d.%m.%Y"))), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$date_num", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(as.numeric(as.Date(c("15.01.2024", "30.06.2024", "25.12.2024"), "%d.%m.%Y"))), 2)
  )
})

# Window functions

test_that("ds.mutate permits lag()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = lag(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(lag(mtcars$mpg), na.rm = TRUE), 2)
  )
})

test_that("ds.mutate permits cumsum()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = cumsum(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(cumsum(mtcars$mpg)), 2)
  )
})

# Logical and conditional functions

test_that("ds.mutate permits is.na()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("na_df", tidy_expr = list(new_col = is.na(x)), newobj = "res", datasources = conns)
  expect_equal(
    ds.class("res$new_col", datasources = conns)[[1]],
    "logical"
  )
})

test_that("ds.mutate permits if_else()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = if_else(mpg > 20, 1, 0)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 4),
    round(mean(if_else(mtcars$mpg > 20, 1, 0)), 4)
  )
})

test_that("ds.mutate permits case_when()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = case_when(cyl == 4 ~ "small", cyl == 6 ~ "medium", cyl == 8 ~ "large")), newobj = "res", datasources = conns)
  expect_equal(
    ds.class("res$new_col", datasources = conns)[[1]],
    "character"
  )
})

# Other functions

test_that("ds.mutate permits desc()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = desc(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    ds.class("res$new_col", datasources = conns)[[1]],
    "numeric"
  )
})

test_that("ds.mutate permits scale()", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("mtcars", tidy_expr = list(new_col = scale(mpg)), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(scale(mtcars$mpg)), 2)
  )
})

# Nested function calls (regression test for c( regex)

test_that("ds.mutate permits as.numeric(as.Date()) nested call", {
  skip_if_not_installed("dsBaseClient")
  ds.mutate("date_df", tidy_expr = list(new_col = as.numeric(as.Date(date_str))), newobj = "res", datasources = conns)
  expect_equal(
    round(ds.mean("res$new_col", datasources = conns)$Mean.by.Study[1, 1], 2),
    round(mean(as.numeric(as.Date(c("2024-01-15", "2024-06-30", "2024-12-25")))), 2)
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

login_data <- .prepare_dslite(assign_method = "bindColsDS", tables = list(mtcars = mtcars))
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")

test_that("ds.bind_cols binds two data frames together", {
  skip_if_not_installed("dsBaseClient")
  ds.bind_cols(
    to_combine = list(mtcars, mtcars),
    newobj = "cols_bound",
    datasources = conns
  )

  expect_equal(
    ds.class("cols_bound", datasources = conns)[[1]],
    "data.frame"
  )

  expect_equal(
    ds.dim("cols_bound", datasources = conns)[[1]],
    c(32, 22)
  )

  expect_equal(
    ds.colnames("cols_bound", datasources = conns)[[1]],
    c(
      "mpg...1", "cyl...2", "disp...3", "hp...4", "drat...5", "wt...6", "qsec...7", "vs...8",
      "am...9", "gear...10", "carb...11", "mpg...12", "cyl...13", "disp...14", "hp...15",
      "drat...16", "wt...17", "qsec...18", "vs...19", "am...20", "gear...21", "carb...22"
    )
  )
})

test_that("ds.bind_cols works with .namerepair arg", {
  skip_if_not_installed("dsBaseClient")
  ds.bind_cols(
    to_combine = list(mtcars, mtcars),
    .name_repair = "universal",
    newobj = "cols_bound",
    datasources = conns
  )

  expect_equal(
    ds.colnames("cols_bound", datasources = conns)[[1]],
    c(
      "mpg...1", "cyl...2", "disp...3", "hp...4", "drat...5", "wt...6", "qsec...7", "vs...8",
      "am...9", "gear...10", "carb...11", "mpg...12", "cyl...13", "disp...14", "hp...15",
      "drat...16", "wt...17", "qsec...18", "vs...19", "am...20", "gear...21", "carb...22"
    )
  )

  ds.bind_cols(
    to_combine = list(mtcars, mtcars),
    .name_repair = "minimal",
    newobj = "cols_bound",
    datasources = conns
  )

  expect_equal(
    ds.colnames("cols_bound", datasources = conns)[[1]],
    c(
      "mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb",
      "mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb"
    )
  )
})

test_that("ds.bind_cols throws error with .namerepair as 'check_unique'", {
  expect_error(
    ds.bind_cols(
      to_combine = list(mtcars, mtcars),
      .name_repair = "check_unique",
      newobj = "cols_bound",
      datasources = conns
    )
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

mtcars <- mtcars %>% mutate(cat_var = factor(ifelse(mpg > 20, "high", "low")))
login_data <- .prepare_dslite(assign_method = "caseWhenDS", tables = list(mtcars = mtcars))
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")

test_that("ds.case_when passes and numeric condition and numeric output", {
  skip_if_not_installed("dsBaseClient")
  ds.case_when(
    tidy_expr = list(
      mtcars$mpg < 20 ~ "low",
      mtcars$mpg >= 20 & mtcars$mpg < 30 ~ "medium",
      mtcars$mpg >= 30 ~ "high"
    ),
    newobj = "test",
    datasources = conns
  )

  nqmes <- names(ds.table("test", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts)

  expect_equal(
    nqmes,
    c("high", "low", "medium", "NA")
  )

  expect_equal(
    ds.class("test", datasources = conns)[[1]],
    "character"
  )

  counts <- ds.table("test", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts
  expect_equal(
    as.numeric(counts, datasources = conns),
    c(12, 54, 30, 0)
  )
})

test_that("ds.case_when correctly passes argument with numeric condition and numeric outcome", {
  skip_if_not_installed("dsBaseClient")
  ds.case_when(
    tidy_expr = list(
      mtcars$mpg < 20 ~ 20,
      mtcars$mpg >= 20 & mtcars$mpg < 30 ~ 30,
      mtcars$mpg >= 30 ~ 40
    ),
    newobj = "test",
    datasources = conns
  )

  nqmes <- names(ds.table("test", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts)

  expect_equal(
    nqmes,
    c("20", "30", "40", "NA")
  )

  expect_equal(
    ds.class("test", datasources = conns)[[1]],
    "numeric"
  )

  counts <- ds.table("test", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts
  expect_equal(
    as.numeric(counts, datasources = conns),
    c(54, 30, 12, 0)
  )
})


test_that("ds.case_when correctly passes argument with categorical condition and categorical outcome", {
  skip_if_not_installed("dsBaseClient")
  ds.case_when(
    tidy_expr = list(
      mtcars$gear == 2 ~ "low",
      mtcars$gear == 3 ~ "medium",
      mtcars$gear == 4 ~ "high",
      mtcars$gear == 5 ~ "very_high"
    ),
    newobj = "test",
    datasources = conns
  )

  nqmes <- names(ds.table("test", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts)

  expect_equal(
    nqmes,
    c("high", "medium", "very_high", "NA")
  )

  expect_equal(
    ds.class("test", datasources = conns)[[1]],
    "character"
  )

  counts <- ds.table("test", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts
  expect_equal(
    as.numeric(counts, datasources = conns),
    c(36, 45, 15, 0)
  )
})

test_that("ds.case_when correctly passes .default argument", {
  skip_if_not_installed("dsBaseClient")
  ds.case_when(
    tidy_expr = list(
      mtcars$gear == 2 ~ "low",
      mtcars$gear == 3 ~ "medium",
      mtcars$gear == 5 ~ "very_high"
    ),
    .default = "something_missing",
    newobj = "test",
    datasources = conns
  )

  nqmes <- names(ds.table("test", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts)

  expect_equal(
    nqmes,
    c("medium", "something_missing", "very_high", "NA")
  )

  expect_equal(
    ds.class("test", datasources = conns)[[1]],
    "character"
  )

  counts <- ds.table("test", datasources = conns)$output.list$TABLES.COMBINED_all.sources_counts
  expect_equal(
    as.numeric(counts, datasources = conns),
    c(45, 36, 15, 0)
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

login_data <- .prepare_dslite(assign_method = "filterDS", tables = list(mtcars = mtcars))
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")

test_that("ds.filter correctly filters", {
  skip_if_not_installed("dsBaseClient")
  ds.filter(
    df.name = "mtcars",
    tidy_expr = list(cyl == 4 & mpg > 20),
    newobj = "filtered",
    datasources = conns
  )

  expect_equal(
    ds.class("filtered", datasources = conns)[[1]],
    "data.frame"
  )

  expect_equal(
    ds.dim("filtered", datasources = conns)[[1]],
    c(11, 11)
  )

  expect_equal(
    ds.mean("filtered$mpg", datasources = conns)$Mean.by.Study[[1]],
    26.66364,
    tolerance = 0.001
  )
})

test_that("ds.filter works with .by arg", {
  skip_if_not_installed("dsBaseClient")
  ds.filter(
    df.name = "mtcars",
    tidy_expr = list(mpg > median(mpg)),
    .by = "cyl",
    newobj = "filtered_by",
    datasources = conns
  )

  expect_equal(
    ds.dim("filtered_by", datasources = conns)[[1]],
    c(14, 11)
  )

  expect_equal(
    ds.mean("filtered_by$mpg", datasources = conns)$Mean.by.Study[[1]],
    22.90714,
    tolerance = 0.00001
  )
})

test_that("ds.filter works with .preserve arg", {
  skip_if_not_installed("dsBaseClient")
  ds.filter(
    df.name = "mtcars",
    tidy_expr = list(mpg > median(mpg)),
    .preserve = T,
    newobj = "preserved_t",
    datasources = conns
  )

  expect_equal(
    ds.class("preserved_t", datasources = conns)[[1]],
    "data.frame"
  )

  ds.filter(
    df.name = "mtcars",
    tidy_expr = list(mpg > median(mpg)),
    .preserve = F,
    newobj = "preserved_f",
    datasources = conns
  )

  expect_equal(
    ds.class("preserved_f", datasources = conns)[[1]],
    "data.frame"
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

data("mtcars")
mtcars_dup_names <- cbind(mtcars, tibble(cyl = 2))

test_matrix <- matrix(data = 1:20, ncol = 4)

login_data <- .prepare_dslite(
  assign_method = "asTibbleDS",
  tables = list(
    mtcars = mtcars,
    mtcars_dup_names = mtcars_dup_names,
    test_matrix = test_matrix
  )
)

conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")
datashield.assign.table(conns, "mtcars_dup_names", "mtcars_dup_names")
datashield.assign.table(conns, "test_matrix", "test_matrix")

test_that("ds.as_tibble correctly converts a data frame to a tibble", {
  skip_if_not_installed("dsBaseClient")
  ds.as_tibble(
    x = "mtcars",
    newobj = "mtcars_tib",
    datasources = conns
  )

  expect_equal(
    ds.class("mtcars_tib", datasources = conns)[[1]],
    c("tbl_df", "tbl", "data.frame")
  )

  expect_equal(
    ds.dim("mtcars_tib", datasources = conns)[[1]],
    c(32, 11)
  )

  expect_equal(
    ds.colnames("mtcars_tib", datasources = conns)[[1]],
    c("mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb")
  )
})

test_that("ds.as_tibble works with the name_repair argument", {
  skip_if_not_installed("dsBaseClient")
  ds.as_tibble(
    x = "mtcars_dup_names",
    .name_repair = "minimal",
    newobj = "mtcars_tib_nr",
    datasources = conns
  )

  expect_equal(
    ds.class("mtcars_tib_nr", datasources = conns)[[1]],
    c("tbl_df", "tbl", "data.frame")
  )

  expect_equal(
    ds.colnames("mtcars_tib_nr", datasources = conns)[[1]],
    c("mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb", "cyl")
  )

  ds.as_tibble(
    x = "mtcars_dup_names",
    .name_repair = "unique",
    newobj = "mtcars_tib_nr",
    datasources = conns
  )

  expect_equal(
    ds.class("mtcars_tib_nr", datasources = conns)[[1]],
    c("tbl_df", "tbl", "data.frame")
  )

  expect_equal(
    ds.colnames("mtcars_tib_nr", datasources = conns)[[1]],
    c("mpg", "cyl...2", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb", "cyl...12")
  )

  expect_error(
    ds.as_tibble(
      x = "mtcars_dup_names",
      .name_repair = "check_unique",
      newobj = "mtcars_tib_nr",
      datasources = conns
    ),
  )

  ds.as_tibble(
    x = "mtcars_dup_names",
    .name_repair = "universal",
    newobj = "mtcars_tib_nr",
    datasources = conns
  )

  expect_equal(
    ds.class("mtcars_tib_nr", datasources = conns)[[1]],
    c("tbl_df", "tbl", "data.frame")
  )

  expect_equal(
    ds.colnames("mtcars_tib_nr", datasources = conns)[[1]],
    c("mpg", "cyl...2", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb", "cyl...12")
  )
})


test_that("ds.as_tibble works with the rownames argument", {
  skip_if_not_installed("dsBaseClient")
  ds.as_tibble(
    x = "mtcars",
    rownames = NULL,
    newobj = "mtcars_tib",
    datasources = conns
  ) ## Not really possible to test clientside as no datashield function to return row names.

  expect_equal(
    ds.class("mtcars_tib", datasources = conns)[[1]],
    c("tbl_df", "tbl", "data.frame")
  )

  ds.as_tibble(
    x = "mtcars",
    rownames = NA,
    newobj = "mtcars_tib",
    datasources = conns
  ) ## Same - can't test other than the object is created.

  expect_equal(
    ds.class("mtcars_tib", datasources = conns)[[1]],
    c("tbl_df", "tbl", "data.frame")
  )

  ds.as_tibble(
    x = "mtcars",
    rownames = "col_with_row_names",
    newobj = "mtcars_tib",
    datasources = conns
  )

  expect_equal(
    ds.class("mtcars_tib", datasources = conns)[[1]],
    c("tbl_df", "tbl", "data.frame")
  )

  expect_equal(
    ds.colnames("mtcars_tib", datasources = conns)[[1]],
    c("col_with_row_names", "mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb")
  )

  expect_equal(
    ds.class("mtcars_tib$col_with_row_names", datasources = conns)[[1]],
    "character"
  )
})

test_that("ds.as_tibble works with matrices", {
  skip_if_not_installed("dsBaseClient")
  ds.as_tibble(
    x = "test_matrix",
    newobj = "mtcars_tib",
    .name_repair = "minimal",
    datasources = conns
  )

  expect_equal(
    ds.class("mtcars_tib", datasources = conns)[[1]],
    c("tbl_df", "tbl", "data.frame")
  )

  expect_equal(
    ds.colnames("mtcars_tib", datasources = conns)[[1]],
    c("", "", "", "")
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

data("mtcars")
login_data <- .prepare_dslite(assign_method = "bindRowsDS", tables = list(mtcars = mtcars))
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")

test_that("ds.rename fails with correct error message if data not present ", {
  skip_if_not_installed("dsBaseClient")
  expect_error(
    ds.rename(
      df.name = "datanotthere",
      tidy_expr = list(test_1 = mpg, test_2 = drat),
      newobj = "nodata",
      datasources = conns
    )
  )
})

test_that("ds.rename correctly passes =", {
  skip_if_not_installed("dsBaseClient")
  ds.rename(
    df.name = "mtcars",
    tidy_expr = list(test_1 = mpg, test_2 = drat),
    newobj = "mpg_drat",
    datasources = conns
  )

  expect_equal(
    ds.colnames("mpg_drat", datasources = conns)[[1]],
    c("test_1", "cyl", "disp", "hp", "test_2", "wt", "qsec", "vs", "am", "gear", "carb")
  )
})

test_that("ds.rename throws an error if column name doesn't exist", {
  skip_if_not_installed("dsBaseClient")
  expect_error(
    ds.rename(
      df.name = "mtcars",
      tidy_expr = list(test_1 = doesntexist, test_2 = drat),
      newobj = "mpg_drat",
      datasources = conns
    )
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

data("mtcars")
mtcars_group <- mtcars %>%
  group_by(carb) %>%
  mutate(drop_test = factor("a", levels = c("a", "b")))

login_data <- .prepare_dslite(assign_method = "sliceDS", tables = list(mtcars = mtcars))
conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")
datashield.assign.table(conns, "mtcars_group", "mtcars_group")

test_that("ds.slice correctly subsets rows", {
  skip_if_not_installed("dsBaseClient")
  ds.slice(
    df.name = "mtcars",
    tidy_expr = list(1:5),
    newobj = "sliced",
    datasources = conns
  )

  expect_equal(
    ds.class("sliced", datasources = conns)[[1]],
    "data.frame"
  )

  expect_equal(
    ds.dim("sliced", datasources = conns)[[1]],
    c(5, 11)
  )
})

test_that("ds.slice works with .by arg", {
  skip_if_not_installed("dsBaseClient")
  ds.slice(
    df.name = "mtcars",
    tidy_expr = list(1:5),
    .by = "cyl",
    newobj = "sliced_by",
    datasources = conns
  )

  expect_equal(
    ds.dim("sliced_by", datasources = conns)[[1]],
    c(15, 11)
  )
}) ## Currently not possible to test .preserve clientside because ds.group_keys not in this PR. Will
## test serverside

test_that("ds.slice throws error if disclosure risk", {
  skip_if_not_installed("dsBaseClient")
  expect_error(
    ds.slice(
      df.name = "mtcars",
      tidy_expr = list(1),
      newobj = "sliced_by",
      datasources = conns
    )
  )
})

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

mtcars_group <- mtcars %>%
  group_by(cyl) %>%
  mutate(drop_test = factor("a", levels = c("a", "b")))

login_data <- .prepare_dslite(
  aggregate_method = "groupKeysDS",
  tables = list(
    mtcars = mtcars,
    mtcars_group = mtcars_group
  )
)

conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars_group", "mtcars_group")
datashield.assign.table(conns, "mtcars_bad_group", "mtcars_bad_group")

test_that("ds.group_keys correctly returns groups", {
  skip_if_not_installed("dsBaseClient")
  groups <- ds.group_keys("mtcars_group", datasources = conns)

  expect_equal(
    groups[[1]],
    tibble(cyl = c(4, 6, 8))
  )
})

test_that("ds.group_keys returns error if too many groups", {
  skip_if_not_installed("dsBaseClient")
  expect_error(
    ds.group_keys("mtcars_bad_group", datasources = conns)
  )
})

#' @title Keep or drop columns using their names and types
#' @description DataSHIELD implementation of \code{dplyr::select}.
#' @param df.name Character specifying a server-side data frame or tibble.
#' @param tidy_expr List of one or more unquoted expressions separated by commas. Variable names can
#' be used as if they were positions in the data frame, so expressions like x:y can be used to
#' select a range of variables.
#' @param newobj Character specifying name for new server-side data frame.
#' @param datasources DataSHIELD connections object.
#' @return No return value, called for its side effects. An object (typically a data frame or tibble)
#' with the name specified by \code{newobj} is created on the server.
#' @importFrom DSI datashield.assign
#' @importFrom rlang enquo
#' @examples
#' \dontrun{
#' ds.select(
#'   df.name = "mtcars",
#'   tidy_expr = list(mpg, starts_with("t")),
#'   newobj = "df_subset",
#'   dataources = conns
#' )
#' }
#' @export
ds.select <- function(df.name = NULL, tidy_expr = NULL, newobj = NULL, datasources = NULL) {
  tidy_expr <- .format_args_as_string(enquo(tidy_expr))
  datasources <- .set_datasources(datasources)
  cally <- .make_serverside_call("selectDS", tidy_expr, list(df.name))
  datashield.assign(datasources, newobj, cally)
}

#' @title Coerce a data frame or matrix to a tibble.
#' @description DataSHIELD implementation of \code{tibble::as_tibble}. Currently only implemented
#' for data frames and tibbles.
#' @param x A data frame or matrix.
#' @param .rows The number of rows, useful to create a 0-column tibble or just as an additional
#' check.
#' @param .name_repair Treatment of problematic column names:
#'   \itemize{
#'     \item "minimal": No name repair or checks, beyond basic existence.
#'     \item "unique": Make sure names are unique and not empty.
#'     \item "check_unique": (default value), no name repair, but check they are unique.
#'     \item "universal": Make the names unique and syntactic.
#'   }
#' @param rownames How to treat existing row names of a data frame or matrix:
#'   \itemize{
#'     \item `NULL`: remove row names. This is the default.
#'     \item `NA`: keep row names.
#'     \item A string: the name of a new column. Existing rownames are transferred
#'     into this column and the \code{row.names} attribute is deleted.
#'     No name repair is applied to the new column name, even if `x` already contains
#'     a column of that name.
#'   }
#' @param newobj Character specifying name for new server-side data frame.
#' @param datasources DataSHIELD connections object.
#' @return No return value, called for its side effects. A tibble with the name specified by \code{newobj} is created on the server.
#' @importFrom DSI datashield.assign
#' @examples
#' \dontrun{
#' ds.as_tibble(
#'   x = "mtcars",
#'   newobj = "mtcars_tib",
#'   datasources = conns
#' )
#' }
#' @export
ds.as_tibble <- function(x = NULL, .rows = NULL, .name_repair = "check_unique", rownames = NULL,
                         newobj = NULL, datasources = NULL) {
  datasources <- .set_datasources(datasources)
  .check_tidy_args(x, newobj)
  cally <- .make_serverside_call("asTibbleDS", NULL, list(x, .rows, .name_repair, rownames))
  datashield.assign(datasources, newobj, cally)
}

#' @title Group by one or more variables
#' @description DataSHIELD implementation of \code{dplyr::group_by}.
#' @param df.name Character specifying a server-side data frame or tibble.
#' @param tidy_expr List of variables or computations to group by.
#' @param .add 	When FALSE, the default, \code{group_by()} will override existing groups. To add to the
#' existing groups, use .add = TRUE.
#' @param .drop Drop groups formed by factor levels that don't appear in the data? The default is
#' TRUE except when .data has been previously grouped with .drop = FALSE.
#' @param newobj Character specifying name for new server-side data frame.
#' @param datasources DataSHIELD connections object.
#' @return No return value, called for its side effects. A grouped data frame with class grouped_df
#' \code{newobj} is created on the server, unless the combination of \code{tidy_expr} and \code{.add}
#' yields a empty set of grouping columns, in which case a tibble will be created on the server.
#' @importFrom DSI datashield.assign
#' @importFrom rlang enquo
#' @examples
#' \dontrun{
#' ds.group_by(
#'   df.name = "mtcars",
#'   expr = list(mpg, cyl),
#'   newobj = "grouped_df"
#' )
#' }
#' @export
ds.group_by <- function(df.name = NULL, tidy_expr, .add = FALSE, .drop = TRUE, newobj = NULL, datasources = NULL) {
  tidy_expr <- .format_args_as_string(enquo(tidy_expr))
  datasources <- .set_datasources(datasources)
  .check_tidy_args(df.name, newobj)
  cally <- .make_serverside_call("groupByDS", tidy_expr, list(df.name, .add, .drop))
  datashield.assign(datasources, newobj, cally)
}

#' @title Remove grouping from a tibble or data frame
#' @description DataSHIELD implementation of \code{dplyr::ungroup}.
#' @param x a tibble or data frame.
#' @param newobj Character specifying name for new server-side data frame.
#' @param datasources DataSHIELD connections object.
#' @return No return value, called for its side effects. An ungrouped data frame or tibble is
#' created on the server.
#' @examples
#' \dontrun{
#' ds.ungroup("grouped_df")
#' }
#' @export
ds.ungroup <- function(x = NULL, newobj = NULL, datasources = NULL) {
  datasources <- .set_datasources(datasources)
  .check_tidy_args(x, newobj)
  cally <- .make_serverside_call("ungroupDS", tidy_select = NULL, list(x))
  datashield.assign(datasources, newobj, cally)
}

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

mtcars_group <- mtcars %>%
  group_by(cyl) %>%
  mutate(drop_test = factor("a", levels = c("a", "b")))

login_data <- .prepare_dslite(
  assign_method = "groupByDS",
  tables = list(
    mtcars = mtcars,
    mtcars_group = mtcars_group
  )
)

conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars", "mtcars")
datashield.assign.table(conns, "mtcars_group", "mtcars_group")

test_that("ds.group_by correctly groups a data frame", {
  skip_if_not_installed("dsBaseClient")
  ds.group_by(
    df.name = "mtcars",
    tidy_expr = list(mpg, cyl),
    newobj = "grouped",
    datasources = conns
  )

  expect_equal(
    ds.class("grouped", datasources = conns)[[1]],
    c("grouped_df", "tbl_df", "tbl", "data.frame")
  )
})

login_data <- .prepare_dslite(
  assign_method = "ungroupDS",
  tables = list(mtcars_group = mtcars_group)
)

conns <- datashield.login(logins = login_data)
datashield.assign.table(conns, "mtcars_group", "mtcars_group")

test_that("ds.ungroup correctly ungroups a data frame", {
  skip_if_not_installed("dsBaseClient")
  ds.ungroup("mtcars_group", "ungrouped_df", datasources = conns)

  expect_equal(
    ds.class("ungrouped_df", datasources = conns)[[1]],
    c("tbl_df", "tbl", "data.frame")
  )
})

#' @title Rename columns
#' @description DataSHIELD implementation of \code{dplyr::rename}.
#' @param df.name Character specifying a server-side data frame or tibble.
#' @param tidy_expr List with format new_name = old_name to rename selected variables.
#' @param newobj Character specifying name for new server-side data frame.
#' @param datasources DataSHIELD connections object.
#' @return No return value, called for its side effects. An object (typically a data frame or tibble)
#' with the name specified by \code{newobj} is created on the server.
#' @importFrom DSI datashield.assign
#' @importFrom rlang enquo
#' @examples
#' \dontrun{
#' ## First log in to a DataSHIELD session with mtcars dataset loaded.
#'
#' ds.rename(
#'   df.name = "mtcars",
#'   tidy_select = list(new_var_1 = mpg, new_var_2 = cyl),
#'   newobj = "df_renamed",
#'   dataources = conns
#' )
#'
#' ## Refer to the package vignette for more examples.
#' }
#' @export
ds.rename <- function(df.name = NULL, tidy_expr = NULL, newobj = NULL, datasources = NULL) {
  tidy_expr <- .format_args_as_string(enquo(tidy_expr))
  datasources <- .set_datasources(datasources)
  .check_tidy_args(df.name, newobj)
  cally <- .make_serverside_call("renameDS", tidy_expr, list(df.name))
  datashield.assign(datasources, newobj, cally)
}

#' Retrieve datasources if not specified
#'
#' @param datasources An optional list of data sources. If not provided, the function will attempt
#' to find available data sources.
#' @importFrom DSI datashield.connections_find
#' @return A list of data sources.
#' @noRd
.get_datasources <- function(datasources) {
  if (is.null(datasources)) {
    datasources <- datashield.connections_find()
  }
  return(datasources)
}

#' Verify that the provided data sources are of class 'DSConnection'.
#'
#' @param datasources A list of data sources.
#' @importFrom cli cli_abort
#' @noRd
.verify_datasources <- function(datasources) {
  is_connection_class <- sapply(datasources, function(x) inherits(unlist(x), "DSConnection"))
  if (!all(is_connection_class)) {
    cli_abort("The 'datasources' were expected to be a list of DSConnection-class objects")
  }
}

#' Set and verify data sources.
#'
#' @param datasources An optional list of data sources. If not provided, the function will attempt
#' to find available data sources.
#' @return A list of verified data sources.
#' @noRd
.set_datasources <- function(datasources) {
  datasources <- .get_datasources(datasources)
  .verify_datasources(datasources)
  return(datasources)
}


#' Set a new object or defaults to '.data' if no object is provided.
#'
#' @param newobj An optional new object name. If not provided, the function defaults to '.data'.
#' @return The provided new object name or '.data' if no object is provided.
#' @noRd
.set_new_obj <- function(.data, newobj) {
  if (is.null(newobj)) {
    newobj <- .data
  }
  return(newobj)
}

#' Generate an encoding key which is used for encoding and decoding strings to pass the R parser
#'
#' @return A list containing the encoding key, with 'input' specifying the characters to be encoded
#' and 'output' specifying their corresponding encoded values.
#' @noRd
.get_encode_dictionary <- function() {
  encode_list <- list(
    input = c("(", ")", "\"", ",", " ", "!", "&", "|", "'", "=", "+", "-", "*", "/", "^", ">", "<", "~", "\n", "%"),
    output = c(
      "$LB$", "$RB$", "$QUOTE$", "$COMMA$", "$SPACE$", "$EXCL$", "$AND$", "$OR$",
      "$APO$", "$EQU$", "$ADD$", "$SUB$", "$MULT$", "$DIVIDE$", "$POWER$", "$GT$", "$LT$", "$TILDE$", "$LINE$", "$PCT$"
    )
  )
  return(encode_list)
}

#' Remove List
#'
#' This function removes the 'list(' portion from a string.
#'
#' @param string A string containing the characters 'list('.
#' @return The string with 'list(' removed.
#' @noRd
.remove_list <- function(string) {
  if (string != "NULL") {
    string <- gsub("list\\(", "", string, fixed = FALSE)
    string <- substr(string, 1, nchar(string) - 1)
  }
}

#' Converts expressions to a string and remove the 'list(' portion.
#'
#' @param expr The expression to be converted.
#' @return The formatted arguments as a string.
#' @importFrom rlang quo_text
#' @noRd
.format_args_as_string <- function(expr) {
  neat_args_as_string <- .remove_list(quo_text(expr))
  return(neat_args_as_string)
}

#' Encode a string using the provided encoding key.
#'
#' @param input_string The string to be encoded.
#' @param encode_key The encoding key generated by '.get_encode_dictionary()'.
#' @return The encoded string.
#' @noRd
.encode_tidy_eval <- function(input_string, encode_key) {
  encode_vec <- encode_key$output
  names(encode_vec) <- encode_key$input
  split_string <- strsplit(input_string, "")[[1]]
  output_string <- sapply(split_string, function(char) {
    if (char %in% names(encode_vec)) {
      encode_vec[[char]]
    } else {
      char
    }
  })
  return(paste(output_string, collapse = ""))
}

#' Check Select Arguments
#'
#' @param .data Character specifying a server-side data frame or tibble.
#' @param newobj Optionally, character specifying name for new server-side data frame.
#' @return This function does not return a value but is used for argument validation.
#' @importFrom assertthat assert_that
#' @noRd
.check_tidy_args <- function(df.name = NULL, newobj, check_df = TRUE, check_obj = TRUE) {
  if (check_df) {
    assert_that(is.character(df.name))
  }

  if (check_obj) {
    assert_that(is.character(newobj))
  }
}

#' Create a Tidy Evaluation Call
#'
#' This function constructs a call object for a tidy evaluation function.
#' It allows for the dynamic creation of function calls in a tidyverse-compatible manner.
#'
#' @param tidy_select Encoded tidyselect arguments
#' @param fun_name The name of the function to be called (as a string), e.g., "select", "mutate".
#' @param other_args A list of additional arguments to be passed to the function (optional).
#' @return A call object that can be evaluated to perform the specified operation.
#' @noRd
.make_serverside_call <- function(fun_name, tidy_select, other_args) {
  if (!is.null(tidy_select)) {
    tidy_select <- .encode_tidy_eval(tidy_select, .get_encode_dictionary())
  }
  cally <- .build_cally(fun_name, c(list(tidy_select), other_args))
  return(cally)
}

#' Construct a Call Object
#'
#' This function constructs and returns a call object based on the provided
#' function name, dataframe name, tidyselect specification, and additional arguments.
#'
#' @param fun_name A character string representing the function name.
#' @param df.name The name of the dataframe.
#' @param tidy_select Tidyselect specification (e.g., column names or selection helpers).
#' @param other_args Additional arguments to be included in the call. If NULL, no additional arguments are added.
#' @importFrom rlang sym
#' @return A call object constructed from the provided arguments.
#' @noRd
.build_cally <- function(fun_name, other_args) {
  arg_list <- c(list(sym(fun_name)), other_args)
  return(as.call(arg_list))
}

require(DSLite)
require(DSI)
require(dplyr)
require(dsTidyverse)
require(dsBaseClient)

data("mtcars")
login_data <- .prepare_dslite()
conns <- datashield.login(logins = login_data)

test_that(".get_datasources returns datasources found by datashield.connections_find() when datasources is NULL", {
  expect_equal(
    with_mocked_bindings(
      .get_datasources(datasources = NULL),
      "datashield.connections_find" = function() conns
    ),
    conns
  )
})

test_that(".get_datasources returns provided datasources when datasources is not NULL", {
  expect_equal(
    .get_datasources(datasources = conns),
    conns
  )
})

test_that(".verify_datasources throws an error if datasources contain non-DSConnection objects", {
  datasources <- c(conns, "not a DSConnection")
  expect_error(.verify_datasources(datasources), "The 'datasources' were expected to be a list of DSConnection-class objects")
})

test_that(".verify_datasources does not throw an error if all datasources are DSConnection objects", {
  datasources <- conns
  expect_silent(.verify_datasources(datasources))
})

test_that(".set_datasources returns datasources found by datashield.connections_find() when datasources is NULL", {
  expect_equal(
    with_mocked_bindings(
      .set_datasources(datasources = NULL),
      "datashield.connections_find" = function() conns
    ),
    conns
  )
})

test_that(".set_datasources returns provided datasources when datasources is not NULL", {
  expect_equal(
    .set_datasources(datasources = conns),
    conns
  )
})

test_that(".set_datasources throws an error if datasources contain non-DSConnection objects", {
  datasources <- c(conns, "not a DSConnection")
  expect_error(.set_datasources(datasources), "The 'datasources' were expected to be a list of DSConnection-class objects")
})

test_that(".set_datasources does not throw an error if all datasources are DSConnection objects", {
  datasources <- conns
  expect_silent(.set_datasources(datasources))
})

test_that(".set_new_obj returns new object name when provided", {
  newobj <- "my_new_object"
  result <- .set_new_obj(.data = "my_data", newobj = newobj)
  expect_equal(result, newobj)
})

test_that(".set_new_obj defaults to .data if no new object name is provided", {
  result <- .set_new_obj(.data = "my_data", newobj = NULL)
  expect_equal(result, "my_data")
})

test_that(".get_encode_dictionary returns the expected encoding key", {
  expected_encode_list <- list(
    input = c("(", ")", "\"", ",", " ", "!", "&", "|", "'", "=", "+", "-", "*", "/", "^", ">", "<", "~", "\n"),
    output = c(
      "$LB$", "$RB$", "$QUOTE$", "$COMMA$", "$SPACE$", "$EXCL$", "$AND$", "$OR$",
      "$APO$", "$EQU$", "$ADD$", "$SUB$", "$MULT$", "$DIVIDE$", "$POWER$",
      "$GT$", "$LT$", "$TILDE$", "$LINE$"
    )
  )
  actual_encode_list <- .get_encode_dictionary()
  expect_equal(actual_encode_list, expected_encode_list)
})

test_that(".get_encode_dictionary returns the expected encoding key", {
  expected_encode_list <- list(
    input = c("list", "(", ")", "\"", ",", " ", "c", ":", "!", "&", "|", "=="),
    output = c("$LIST$", "$LB$", "$RB$", "$QUOTE$", "$COMMA$", "$SPACE$", "$C$", "$COLON$", "$EXCL$", "$AND$", "$OR$", "$EQUALS$")
  )
  actual_encode_list <- .get_encode_dictionary()
  expect_false(isTRUE(all.equal(expected_encode_list, actual_encode_list)))
})

test_that(".remove_list removes 'list(' portion from a string", {
  input_string <- "list(a, b, c)"
  output_string <- .remove_list(input_string)
  expect_equal(output_string, "a, b, c")
})

test_that(".remove_list handles input without 'list(' correctly", {
  input_string <- "c(a, b, c)"
  output_string <- .remove_list(input_string)
  expect_equal(output_string, "c(a, b, c")
})

test_that(".format_args_as_string correctly formats including 'list'", {
  expr <- rlang::expr(
    list(LAB_TSC, starts_with("LAB"), ends_with("ED"))
  )
  expected <- "LAB_TSC, starts_with(\"LAB\"), ends_with(\"ED\")"
  result <- .format_args_as_string(expr)
  expect_equal(result, expected)
})

test_that(".format_args_as_string correctly formats excluding 'list'", {
  expr <- rlang::expr(
    c(LAB_TSC, starts_with("LAB"), ends_with("ED"))
  )
  expected <- "c(LAB_TSC, starts_with(\"LAB\"), ends_with(\"ED\")"
  result <- .format_args_as_string(expr)
  expect_equal(result, expected)
})

test_that(".encode_tidy_eval correctly encodes strings with permitted values", {
  encode_key <- .get_encode_dictionary()
  input_string <- "asd, qwe, starts_with('test')"
  expected_output <- "asd$COMMA$$SPACE$qwe$COMMA$$SPACE$starts_with$LB$$APO$test$APO$$RB$"
  result <- .encode_tidy_eval(input_string, encode_key)
  expect_equal(result, expected_output)
})

test_that(".encode_tidy_eval correctly encodes strings with unpermitted values", {
  encode_key <- .get_encode_dictionary()
  input_string <- "asd, qwe, wer == rew &}{}%"
  expected_output <- "asd$COMMA$$SPACE$qwe$COMMA$$SPACE$wer$SPACE$$EQU$$EQU$$SPACE$rew$SPACE$$AND$}{}%"
  result <- .encode_tidy_eval(input_string, encode_key)
  expect_equal(result, expected_output)
})

test_that(".build_cally build correct call object", {
  input_string <- "LAB_TSC, starts_with(\"LAB\"), ends_with(\"ED\")"
  expected <- call("mutateDS", "LAB_TSC, starts_with(\"LAB\"), ends_with(\"ED\")", "mydata")
  returned <- .build_cally("mutateDS", list(input_string, "mydata"))
  expect_equal(expected, returned)

  expected <- call(
    "mutateDS", "LAB_TSC, starts_with(\"LAB\"), ends_with(\"ED\")", "mydata",
    "extra_arg_1", "extra_arg_2"
  )
  returned <- .build_cally("mutateDS", list(input_string, "mydata", "extra_arg_1", "extra_arg_2"))
  expect_equal(expected, returned)
})

test_that(".make_serverside_call encodes and builds call object", {
  input_string <- "c(LAB_TSC, starts_with(\"LAB\"), ends_with(\"ED\")"
  tidy_select <- .format_args_as_string(rlang::enquo(input_string))
  returned <- .make_serverside_call("mutateDS", tidy_select, list("mydata"))
  expected <- call("mutateDS", "$QUOTE$c$LB$LAB_TSC$COMMA$$SPACE$starts_with$LB$\\$QUOTE$LAB\\$QUOTE$$RB$$COMMA$$SPACE$ends_with$LB$\\$QUOTE$ED\\$QUOTE$$RB$", "mydata")
  expect_equal(expected, returned)

  returned <- .make_serverside_call("mutateDS", tidy_select, list("mydata", "extra_arg_1", "extra_arg_2"))
  expected <- call(
    "mutateDS", "$QUOTE$c$LB$LAB_TSC$COMMA$$SPACE$starts_with$LB$\\$QUOTE$LAB\\$QUOTE$$RB$$COMMA$$SPACE$ends_with$LB$\\$QUOTE$ED\\$QUOTE$$RB$",
    "mydata", "extra_arg_1", "extra_arg_2"
  )
  expect_equal(expected, returned)
})

test_that(".check_tidy_args returns correct errors", {
  expect_error(
    .check_tidy_args(df.name = NULL, newobj = "test", check_df = TRUE, check_obj = TRUE),
    "df.name is not a character vector"
  )

  expect_error(
    .check_tidy_args(df.name = "test", newobj = NULL, check_df = TRUE, check_obj = TRUE),
    "newobj is not a character vector"
  )

  expect_silent(
    .check_tidy_args(df.name = "test", newobj = "test", check_df = TRUE, check_obj = TRUE)
  )
})

#' @title Keep rows that match a condition
#' @description DataSHIELD implentation of \code{dplyr::filter}.
#' @param df.name Character specifying a serverside data frame or tibble.
#' @param tidy_expr List of expressions that return a logical value, and are defined in terms of the
#' variables in \code{df.name}.
#' @param .by Optionally, a selection of columns to group by for just this operation, functioning as an alternative to \code{dplyr::group_by}
#' @param .preserve Relevant when the .data input is grouped. If .preserve = FALSE (the default),
#' the grouping structure is recalculated based on the resulting data, otherwise the grouping is
#' kept as is.
#' @param newobj Character specifying name for new server-side data frame.
#' @param datasources DataSHIELD connections object.
#' @return No return value, called for its side effects. An object (typically a data frame or tibble)
#' with the name specified by \code{newobj} is created on the server.
#' @importFrom DSI datashield.assign
#' @importFrom rlang enquo
#' @examples
#' \dontrun{
#' ds.filter(
#'   df.name = "mtcars",
#'   tidy_expr = list(cyl == 4 & mpg > 20),
#'   newobj = "filtered",
#'   datasources = conns
#' )
#' }
#' @export
ds.filter <- function(df.name = NULL, tidy_expr = NULL, .by = NULL, .preserve = FALSE, newobj = NULL, datasources = NULL) {
  tidy_expr <- .format_args_as_string(enquo(tidy_expr))
  datasources <- .set_datasources(datasources)
  .check_tidy_args(df.name, newobj)
  cally <- .make_serverside_call("filterDS", tidy_expr, list(df.name, .by, .preserve))
  datashield.assign(datasources, newobj, cally)
}

#' @title Order the rows of a data frame by the values of selected columns
#' @description DataSHIELD implentation of \code{dplyr::arrange}.
#' @param df.name Character specifying a serverside data frame or tibble.
#' @param tidy_expr A list containing variables, or functions of variables. Use \code{dplyr::desc()} to sort a
#' variable in descending order.
#' @param .by_group If TRUE, will sort first by grouping variable. Applies to grouped data frames
#' only.
#' @param newobj Character specifying name for new server-side data frame.
#' @param datasources DataSHIELD connections object.
#' @return No return value, called for its side effects. An object (typically a data frame or tibble) with the name specified by \code{newobj} is created on the server.
#' @importFrom DSI datashield.assign
#' @importFrom rlang enquo
#' @examples
#' \dontrun{
#' ds.arrange(
#'   df.name = "mtcars",
#'   tidy_expr = list(drat),
#'   newobj = "sorted_df",
#'   datasources = conns
#' )
#' }
#' @export
ds.arrange <- function(df.name = NULL, tidy_expr = NULL, .by_group = NULL, newobj = NULL, datasources = NULL) {
  tidy_expr <- .format_args_as_string(enquo(tidy_expr))
  datasources <- .set_datasources(datasources)
  .check_tidy_args(df.name, newobj)
  cally <- .make_serverside_call("arrangeDS", tidy_expr, list(df.name, .by_group))
  datashield.assign(datasources, newobj, cally)
}

#' @title Create, modify, and delete columns
#' @description DataSHIELD implementation of \code{dplyr::mutate}.
#' @param df.name Character specifying a server-side data frame or tibble.
#' @param tidy_expr List of tidyselect syntax to be passed to dplyr::mutate.
#' @param .keep Control which columns from \code{df.name} are retained in the output. Options include:
#' \itemize{
#'   \item \code{"all"}: Retains all columns from \code{df.name}. This is the default.
#'   \item \code{"used"}: Retains only the columns used in \code{tidy_expr} to create new columns.
#'   \item \code{"unused"}: Retains only the columns not used in \code{tidy_expr} to create new columns. This is useful if you generate new columns but no longer need the columns used to generate them.
#'   \item \code{"none"}: Doesn't retain any extra columns from \code{df.name}. Only the grouping variables and columns created by \code{tidy_expr} are kept.
#' }
#' Grouping columns and columns created by \code{tidy_expr} are always kept.
#' @param .before <tidy-select> Optionally, control where new columns should appear (the default is
#' to add to the right hand side). See \code{tidy_expr} for more details.
#' @param .after <tidy-select> Optionally, control where new columns should appear (the default is
#' to add to the right hand side). See \code{tidy_expr} for more details.
#' @param newobj Character specifying name for new server-side data frame.
#' @param datasources datashield connections object.
#' @return No return value, called for its side effects. An object (typically a data frame or tibble)
#' with the name specified by \code{newobj} is created on the server.
#' @importFrom DSI datashield.assign
#' @importFrom rlang enquo
#' @examples
#' \dontrun{
#' ## First log in to a DataSHIELD session with mtcars dataset loaded.
#'
#' ds.mutate(
#'   df.name = "mtcars",
#'   tidy_select = list(mpg_trans = cyl * 1000, new_var = (hp - drat) / qsec),
#'   newobj = "df_with_new_cols"
#' )
#'
#' ## Refer to the package vignette for more examples.
#' }
#' @export
ds.mutate <- function(df.name = NULL, tidy_expr = NULL, newobj = NULL, .keep = "all", .before = NULL,
                      .after = NULL, datasources = NULL) {
  tidy_expr <- .format_args_as_string(enquo(tidy_expr))
  datasources <- .set_datasources(datasources)
  .check_tidy_args(df.name, newobj)
  cally <- .make_serverside_call("mutateDS", tidy_expr, list(df.name, .keep, .before, .after))
  datashield.assign(datasources, newobj, cally)
}

#' @title Describe the groups within a grouped tibble or data frame
#' @description DataSHIELD implentation of \code{dplyr::group_keys}.
#' @param df.name Character specifying a serverside tibble.
#' @param datasources DataSHIELD connections object.
#' @return A data frame describing the groups.
#' @importFrom DSI datashield.aggregate
#' @examples
#' \dontrun{
#' my_groups <- ds.group_keys("grouped_df")
#' }
#' @export
ds.group_keys <- function(df.name = NULL, datasources = NULL) {
  datasources <- .set_datasources(datasources)
  .check_tidy_args(df.name, NULL, check_obj = F)
  cally <- .make_serverside_call("groupKeysDS", tidy_select = NULL, list(df.name))
  datashield.aggregate(datasources, cally)
}

#' @title Keep distinct/unique rows
#' @description DataSHIELD implementation of \code{dplyr::distinct}.
#' @param df.name Character specifying a server-side data frame or tibble.
#' @param tidy_expr Optionally, list of variables to use when determining uniqueness. If there are multiple rows
#' for a given combination of inputs, only the first row will be preserved. If omitted, will use all
#' variables in the data frame.
#' @param .keep_all If TRUE, keep all variables in .data. If a combination of \code{expr} is not
#' distinct, this keeps the first row of values.
#' @param newobj Character specifying name for new server-side data frame.
#' @param datasources DataSHIELD connections object.
#' @return No return value, called for its side effects. An object (typically a data frame or tibble)
#' with the name specified by \code{newobj} is created on the server.
#' @importFrom DSI datashield.assign
#' @importFrom rlang enquo
#' @examples
#' \dontrun{
#' ds.distinct(
#'   df.name = "mtcars",
#'   expr = list(mpg, cyl),
#'   newobj = "distinct_df"
#' )
#' }
#' @export
ds.distinct <- function(df.name = NULL, tidy_expr = NULL, .keep_all = FALSE, newobj = NULL,
                        datasources = NULL) {
  tidy_expr <- .format_args_as_string(enquo(tidy_expr))
  datasources <- .set_datasources(datasources)
  .check_tidy_args(df.name, newobj)
  cally <- .make_serverside_call("distinctDS", tidy_expr, list(df.name, .keep_all))
  datashield.assign(datasources, newobj, cally)
}

#' @title A general vectorised if-else
#' @description DataSHIELD implementation of \code{dplyr::case_when}.
#' @param tidy_expr A list containing a sequence of two-sided formulas:
#' \itemize{
#'   \item The left hand side (LHS) determines which values match this case.
#'   \item The right hand side (RHS) provides the replacement value.
#'   \item The LHS inputs must evaluate to logical vectors.
#'   \item The RHS inputs will be coerced to their common type.
#'   }
#'   All inputs will be recycled to their common size. We encourage all LHS inputs to be the same size.
#'   Recycling is mainly useful for RHS inputs, where you might supply a size 1 input that will be
#'   recycled to the size of the LHS inputs. NULL inputs are ignored.
#' @param .default The value used when all of the LHS inputs return either FALSE or NA.
#' @param .ptype An optional prototype declaring the desired output type. If supplied, this overrides the common type of true, false, and missing.
#' @param .size An optional size declaring the desired output size. If supplied, this overrides the size of condition.
#' @param newobj Character specifying name for new server-side data frame.
#' @param datasources datashield connections object.
#' @return No return value, called for its side effects. A vector with the same size as the common
#' size computed from the inputs in \code{tidy_expr} and the same type as the common type of the
#' RHS inputs in \code{tidy_expr} is created on the server.
#' @importFrom DSI datashield.assign
#' @importFrom rlang enquo
#' @examples
#' \dontrun{
#' ## First log in to a DataSHIELD session with mtcars dataset loaded.
#'
#' ds.case_when(
#'   tidy_expr = list(
#'     mtcars$mpg < 10 ~ "low",
#'     mtcars$mpg >= 10 & mtcars$mpg < 20 ~ "medium",
#'     mtcars$mpg >= 20 ~ "high"
#'   ),
#'   newobj = "test",
#'   datasources = conns
#' )
#'
#' ## Refer to the package vignette for more examples.
#' }
#' @export
ds.case_when <- function(tidy_expr = NULL, .default = NULL, .ptype = NULL, .size = NULL,
                         newobj = NULL, datasources = NULL) {
  tidy_expr <- .format_args_as_string(enquo(tidy_expr))
  datasources <- .set_datasources(datasources)
  .check_tidy_args(NULL, newobj, check_df = FALSE)
  cally <- .make_serverside_call("caseWhenDS", tidy_expr, list(.default, .ptype, .size))
  datashield.assign(datasources, newobj, cally)
}

#' @title Bind multiple data frames by row.
#' @description DataSHIELD implementation of \code{dplyr::bind_rows}.
#' @param to_combine 	Data frames to combine. Each argument can either be a data frame, a list that
#' could be a data frame, or a list of data frames. Columns are matched by name, and any missing
#' columns will be filled with NA.
#' @param .id The name of an optional identifier column. Provide a string to create an output column
#' that identifies each input. The column will use names if available, otherwise it will use
#' positions.
#' @param newobj Character specifying name for new server-side data frame.
#' @param datasources datashield connections object.
#' @return No return value, called for its side effects. A data frame with the name specified by
#' \code{newobj} and the same type as the first element of \code{to_combine} is created on the
#' server.
#' @importFrom DSI datashield.assign
#' @importFrom rlang enquo
#' @examples
#' \dontrun{
#' ## First log in to a DataSHIELD session with mtcars dataset loaded.
#'
#' ds.bind_rows(
#'   to_combine = list(mtcars, mtcars),
#'   newobj = "test",
#'   datasources = conns
#' )
#'
#' ## Refer to the package vignette for more examples.
#' }
#' @export
ds.bind_rows <- function(to_combine = NULL, .id = NULL, newobj = NULL, datasources = NULL) {
  to_combine <- .format_args_as_string(enquo(to_combine))
  datasources <- .set_datasources(datasources)
  .check_tidy_args(NULL, newobj, check_df = FALSE)
  cally <- .make_serverside_call("bindRowsDS", to_combine, list(.id))
  datashield.assign(datasources, newobj, cally)
}

---
title: "Using tidyverse within DataSHIELD"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using tidyverse within DataSHIELD}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---




## Overview
`dsTidyVerseClient` is the DataSHIELD implementation of selected functions from the Tidyverse (https://tidyverse.org/). Most of these functions are from `dplyr`, with some planned implementations for functions from `purrr` and `tidyr`.

As long as not potentially disclosing, all functionality from the original Tidyverse functions has been preserved. The main argument which is normally passed to the `...` parameter in Tidyverse functions is passed as a list to the parameter `tidy_select`. Other arguments are passed to the relevant parameters which retain the same names as the original function.

To illustrate the usage of these functions, we use DSLite which creates a virtual DataSHIELD session.

## Install and load packages

``` r
## install.packages("dplyr")
## install.packages("DSLite")
## install.packages(c("dsBase", "dsBaseClient"), repos = "https://cran.obiba.org/")
## devtools::install_github("molgenis/dsTidyverse")
## devtools::install_github("molgenis/dsTidyverseClient")
require(DSLite)
require(dplyr)
require(dsBase)
require(dsBaseClient)
require(dsTidyverse)
require(dsTidyverseClient)
require(DSI)
```

## Set up DSLite environment and log in.

``` r

data("mtcars")
mtcars_group <- mtcars %>%
  group_by(cyl) %>%
  mutate(drop_test = factor("a", levels = c("a", "b")))

dslite.server <- newDSLiteServer(
  tables = list(
    mtcars = mtcars,
    mtcars_group = mtcars_group
  )
)

dslite.server$config(defaultDSConfiguration(include=c("dsBase", "dsTidyverse")))
dslite.server$assignMethod("selectDS", "selectDS")
dslite.server$assignMethod("renameDS", "renameDS")
dslite.server$assignMethod("mutateDS", "mutateDS")
dslite.server$assignMethod("ifElseDS", "ifElseDS")
dslite.server$assignMethod("caseWhenDS", "caseWhenDS")
dslite.server$assignMethod("bindRowsDS", "bindRowsDS")
dslite.server$assignMethod("bindColsDS", "bindColsDS")
dslite.server$assignMethod("filterDS", "filterDS")
dslite.server$assignMethod("sliceDS", "sliceDS")
dslite.server$assignMethod("arrangeDS", "arrangeDS")
dslite.server$assignMethod("distinctDS", "distinctDS")
dslite.server$assignMethod("groupByDS", "groupByDS")
dslite.server$assignMethod("ungroupDS", "ungroupDS")
dslite.server$assignMethod("asTibbleDS", "asTibbleDS")

dslite.server$aggregateMethod("groupKeysDS", "groupKeysDS")

builder <- DSI::newDSLoginBuilder()

builder$append(
  server="server_1",
  url="dslite.server",
  table = "mtcars",
  driver = "DSLiteDriver")

logindata <- builder$build()
conns <- DSI::datashield.login(logins = logindata, assign = TRUE)

datashield.assign.table(
  conns = conns,
  table = "mtcars",
  symbol = "mtcars")

datashield.assign.table(
  conns = conns,
  table = "mtcars_group",
  symbol = "mtcars_group")
```

## Functions
### Select
`ds.select`, implements all the flexibility of `dplyr::select` in sub-setting columns. This includes:
(i) the ability to pass unquoted column names which are evaluated as columns of `df.name`, and
(ii) the ability to use tidyselect helper functions, such as `contains`. Below are some examples,
for full details see the documentation for `dplyr::select`.

#### Subset columns by name

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"
ds.select(df.name = "mtcars", tidy_expr = list(mpg, disp, wt), newobj = "new_df")
ds.colnames("new_df")
#> $server_1
#> [1] "mpg"  "disp" "wt"
```
#### Specify columns to exclude

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"
ds.select(df.name = "mtcars", tidy_expr = list(!mpg), newobj = "new_df")
ds.colnames("new_df")
#> $server_1
#>  [1] "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"
```
#### Specify a range of columns to keep

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"
ds.select(df.name = "mtcars", tidy_expr = list(mpg:drat), newobj = "new_df")
ds.colnames("new_df")
#> $server_1
#> [1] "mpg"  "cyl"  "disp" "hp"   "drat"
```
#### Combine multiple conditions with '&'

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"
ds.select(df.name = "mtcars", tidy_expr = list(starts_with('c') & ends_with('b')), newobj = "new_df")
ds.colnames("new_df")
#> $server_1
#> [1] "carb"
```
#### Combine multiple conditions with '|'

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"
ds.select(df.name = "mtcars", tidy_expr = list(starts_with('c') | ends_with('b')), newobj = "new_df")
ds.colnames("new_df")
#> $server_1
#> [1] "cyl"  "carb"
```
### Rename
`ds.rename` renames columns within a server-side dataframe. Column names are passed unquoted and are
evaluated as column names within `df.name`. Below are some examples, see `dplyr::rename` for full details.


``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"
ds.rename(df.name = "mtcars", tidy_expr = list(effiency = mpg, power = hp), newobj = "new_df")
ds.colnames("new_df")
#> $server_1
#>  [1] "effiency" "cyl"      "disp"     "power"    "drat"     "wt"      
#>  [7] "qsec"     "vs"       "am"       "gear"     "carb"
```

### Mutate
`ds.mutate` creates new columns in a server-side dataframe. These new columns are normally transformations of existing columns. This reduces the number of steps currently required in DataSHIELD, i.e. creating
a new vector and joining it back to an existing data frame. Again, column names are passed unquoted.
Below are some examples, see `dplyr::mutate` for full details.

#### Create columns which are transformations of existing variables:

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"
ds.mutate(
  df.name = "mtcars",
  tidy_expr = list(mpg_trans = cyl*1000, new_var = (hp-drat)/qsec),
  newobj = "new_df")
ds.colnames("new_df")
#> $server_1
#>  [1] "mpg"       "cyl"       "disp"      "hp"        "drat"      "wt"       
#>  [7] "qsec"      "vs"        "am"        "gear"      "carb"      "mpg_trans"
#> [13] "new_var"
ds.mean("mtcars$cyl")$Mean.by.Study
#>          EstimatedMean Nmissing Nvalid Ntotal
#> server_1        6.1875        0     32     32
ds.mean("new_df$mpg_trans")$Mean.by.Study
#>          EstimatedMean Nmissing Nvalid Ntotal
#> server_1        6187.5        0     32     32
ds.mean("mtcars$hp")$Mean.by.Study
#>          EstimatedMean Nmissing Nvalid Ntotal
#> server_1      146.6875        0     32     32
ds.mean("mtcars$drat")$Mean.by.Study
#>          EstimatedMean Nmissing Nvalid Ntotal
#> server_1      3.596563        0     32     32
ds.mean("mtcars$qsec")$Mean.by.Study
#>          EstimatedMean Nmissing Nvalid Ntotal
#> server_1      17.84875        0     32     32
ds.mean("new_df$new_var")$Mean.by.Study
#>          EstimatedMean Nmissing Nvalid Ntotal
#> server_1      8.372669        0     32     32
```

#### Choose where the new columns are positioned in the data frame with `.before` and `.after`:

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"
ds.mutate(
  df.name = "mtcars",
  tidy_expr = list(mpg_trans = cyl*1000, new_var = (hp-drat)/qsec),
  newobj = "new_df",
  .before = "disp")
ds.colnames("new_df")
#> $server_1
#>  [1] "mpg"       "cyl"       "mpg_trans" "new_var"   "disp"      "hp"       
#>  [7] "drat"      "wt"        "qsec"      "vs"        "am"        "gear"     
#> [13] "carb"
```

#### Only keep the newly created variables with `.keep`:

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"
ds.mutate(
  df.name = "mtcars",
  tidy_expr = list(mpg_trans = cyl*1000, new_var = (hp-drat)/qsec),
  newobj = "new_df",
  .keep = "none")
ds.colnames("new_df")
#> $server_1
#> [1] "mpg_trans" "new_var"
```

### if_else
A shinier version of `base::ifelse`. As with the other implementations of tidyverse, variable/object
names can be passed unquoted, this time in the `condition` argument.


``` r
ds.if_else(
  condition = list(mtcars$mpg > 20),
  "high",
  "low",
  newobj = "mpg_cat")

ds.table("mpg_cat")$output.list$TABLE_rvar.by.study_counts
#>        study
#> mpg_cat server_1
#>    high       14
#>    low        18
#>    NA          0
```

### bind_rows
Bind any number of data frames by row, making a longer result. This is similar to
do.call(rbind, dfs), but the output will contain all columns that appear in any of the inputs.


``` r
ds.dim("mtcars")[[1]]
#> [1] 32 11

ds.bind_rows(
  to_combine = list(mtcars, mtcars),
  newobj = "df_bound",
  datasources = conns
)

ds.dim("df_bound")[[1]]
#> [1] 64 11
```
The argument `.id` can be used to create an additional column which records which dataframe each
row came from:


``` r
ds.bind_rows(
  to_combine = list(mtcars, mtcars),
  newobj = "df_bound",
  datasources = conns,
  .id = "where_it_came_from"
)

ds.colnames("df_bound")
#> $server_1
#>  [1] "where_it_came_from" "mpg"                "cyl"               
#>  [4] "disp"               "hp"                 "drat"              
#>  [7] "wt"                 "qsec"               "vs"                
#> [10] "am"                 "gear"               "carb"
```

### bind_cols
Bind any number of data frames by column, making a wider result. This is similar to do.call(cbind, dfs).


``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"

ds.bind_cols(
  to_combine = list(mtcars, mtcars),
  newobj = "df_bound",
  datasources = conns
)

ds.colnames("df_bound")
#> $server_1
#>  [1] "mpg...1"   "cyl...2"   "disp...3"  "hp...4"    "drat...5"  "wt...6"   
#>  [7] "qsec...7"  "vs...8"    "am...9"    "gear...10" "carb...11" "mpg...12" 
#> [13] "cyl...13"  "disp...14" "hp...15"   "drat...16" "wt...17"   "qsec...18"
#> [19] "vs...19"   "am...20"   "gear...21" "carb...22"
```
The argument `.name_repair` handles duplicate or broken names, e.g.

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"

ds.bind_cols(
  to_combine = list(mtcars, mtcars),
  newobj = "df_bound",
  .name_repair = "minimal",
  datasources = conns
)

ds.colnames("df_bound")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb" "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"  
#> [21] "gear" "carb"
```
### case_when
An extension of `dplyr::if_else` which allows the specification of multiple conditions. Extremely
useful for recoding variables.


``` r
  ds.case_when(
    tidy_expr = list(
      mtcars$mpg < 20 ~ "low",
      mtcars$mpg >= 20 & mtcars$mpg < 30 ~ "medium",
      mtcars$mpg >= 30 ~ "high"),
    newobj = "recoded",
    datasources = conns)

ds.table("recoded")$output.list$TABLE_rvar.by.study_counts
#>         study
#> recoded  server_1
#>   high          4
#>   low          18
#>   medium       10
#>   NA            0
```

You can also use the `.default` argument to control what happens if the condition is not met:


``` r
  ds.case_when(
    tidy_expr = list(
      mtcars$mpg < 20 ~ "low",
      mtcars$mpg >= 30 ~ "high"),
    newobj = "recoded_missing",
    .default = "something_missing",
    datasources = conns)

ds.table("recoded_missing")$output.list$TABLE_rvar.by.study_counts
#>                    study
#> recoded_missing     server_1
#>   high                     4
#>   low                     18
#>   something_missing       10
#>   NA                       0
```
See the help file for `dplyr::case_when` for all available arguments.

### filter
Subset set a data frame flexibly, retaining all rows that satisfy the condition supplied to
`expression`


``` r
ds.dim("mtcars")[[1]]
#> [1] 32 11
  ds.filter(
    df.name = "mtcars",
    tidy_expr = list(cyl == 4 & mpg > 20),
    newobj = "filtered",
    datasources = conns)
ds.dim("filtered")[[1]]
#> [1] 11 11
```

Use the .by argument to perform the filtering by a specified group.

``` r
ds.dim("mtcars")[[1]]
#> [1] 32 11
  ds.filter(
    df.name = "mtcars",
    tidy_expr = list(mpg > median(mpg)),
    .by = "cyl",
    newobj = "filtered_by",
    datasources = conns)
ds.dim("filtered_by")[[1]]
#> [1] 14 11
```

If filtering on an already grouped tibble, use the `preserve` argument to specify whether to retain
the original groups or recalculate groups based on the resulting data. See the help file for
`dplyr::filter` for more information.


``` r
ds.dim("mtcars")[[1]]
#> [1] 32 11
    ds.filter(
      df.name = "mtcars",
      tidy_expr = list(mpg > median(mpg)),
      .preserve = T,
      newobj = "preserved_t",
      datasources = conns
      )
ds.dim("preserved_t")[[1]]
#> [1] 15 11
```

### Slice
Subset rows using their positions. This is particularly useful if you want to take one observation
within a group (for example if you have repeated measures data, and want to take one measurements
per individual within an age group). Currently this can be done using `dh.createSubset`, however
this should be replaced by `group_by` and `slice` as it will be vastly quicker.


``` r
ds.dim("mtcars")[1]
#> $`dimensions of mtcars in server_1`
#> [1] 32 11
ds.slice(
  df.name = "mtcars",
  tidy_expr = list(1:10),
  newobj = "sliced_df"
)
ds.dim("sliced_df")[1]
#> $`dimensions of sliced_df in server_1`
#> [1] 10 11
```


``` r
ds.dim("mtcars")[1]
#> $`dimensions of mtcars in server_1`
#> [1] 32 11
ds.slice(
  df.name = "mtcars",
  tidy_expr = list(1),
  .by = "cyl",
  newobj = "sliced_df_group"
)
ds.dim("sliced_df_group")[1]
#> $`dimensions of sliced_df_group in server_1`
#> [1]  3 11
```

### Arrange
`arrange` is a more flexible version of `base::sort`, which orders the rows of a data frame by the values of selected columns.


``` r
ds.arrange(
  df.name = "mtcars",
  tidy_expr = list(mpg, cyl),
  newobj = "arranged_df",
  datasources = conns
  )
```
`arrange` can be used on a sorted data frame or tibble. The arrange argument `.by_group` (TRUE or FALSE) determines whether or not data is sorted by group.

### Grouping
Many data operations are performed on groups defined by variables. `group_by` takes an existing tibble or data frame and converts it into a grouped tibble. Subsequent operations can then be performed 'by group' rather than to the whole dataframe.

To group a data frame:

``` r
ds.group_by(
  df.name = "mtcars",
  tidy_expr = list(mpg, cyl),
  newobj = "grouped"
)
ds.class("grouped")[[1]]
#> [1] "grouped_df" "tbl_df"     "tbl"        "data.frame"
```

To ungroup a data frame:

``` r
ds.ungroup("grouped", "no_longer_grouped")
ds.class("no_longer_grouped")[[1]]
#> [1] "tbl_df"     "tbl"        "data.frame"
```

To return to client-side details of the groups, use `ds.group_keys`. Note that
this is subject to disclosure controls and will return an error if the number
of groups is too high.


``` r
my_groups <- ds.group_keys("mtcars_group")
my_groups
#> $server_1
#> # A tibble: 3 × 1
#>     cyl
#>   <dbl>
#> 1     4
#> 2     6
#> 3     8
```

### distinct
Use `distinct` to keep only unique rows. Leave the `expr` argument empty to check uniqueness across all variables:


``` r
ds.distinct(
  df.name = "mtcars",
  newobj = "distinct_df"
)
ds.dim("distinct_df")[[1]]
#> [1] 32 11
```
In this example the dimensions of the resulting dataset are the same because all rows are distinct. Alternatively you can specify a subset of variables in which to check for unique rows:


``` r
ds.distinct(
  df.name = "mtcars",
  tidy_expr = list(cyl, drat),
  newobj = "distinct_subset"
)
ds.dim("distinct_subset")[[1]]
#> [1] 26  2
ds.colnames("distinct_subset")[[1]]
#> [1] "cyl"  "drat"
```
See the help file of `dplyr::distinct` for information about other arguments.


### Convert objects to tibbles
'Tibbles' are the tidyverse version of dataframes. Currently within DataSHIELD it is possible
to convert dataframes and matrices to tibbles:


``` r
ds.class("mtcars")[[1]]
#> [1] "data.frame"
ds.as_tibble(
  x = "mtcars",
  newobj = "mtcars_tib",
  datasources = conns)
  ds.class("mtcars_tib")[[1]]
#> [1] "tbl_df"     "tbl"        "data.frame"
```

---
title: "Using tidyverse within DataSHIELD"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using tidyverse within DataSHIELD}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---




## Overview
`dsTidyVerseClient` is the DataSHIELD implementation of selected functions from the Tidyverse (https://tidyverse.org/). Most of these functions are from `dplyr`, with some planned implementations for functions from `purrr` and `tidyr`.

As long as not potentially disclosing, all functionality from the original Tidyverse functions has been preserved. The main argument which is normally passed to the `...` parameter in Tidyverse functions is passed as a list to the parameter `tidy_select`. Other arguments are passed to the relevant parameters which retain the same names as the original function.

To illustrate the usage of these functions, we use DSLite which creates a virtual DataSHIELD session.

## Install and load packages

``` r
## install.packages("dplyr")
## install.packages("DSLite")
## install.packages(c("dsBase", "dsBaseClient"), repos = "https://cran.obiba.org/")
## devtools::install_github("molgenis/dsTidyverse")
## devtools::install_github("molgenis/dsTidyverseClient")
require(DSLite)
require(dplyr)
require(dsBase)
require(dsBaseClient)
require(dsTidyverse)
require(dsTidyverseClient)
require(DSI)
```

## Set up DSLite environment and log in.

``` r

data("mtcars")
mtcars_group <- mtcars %>%
  group_by(cyl) %>%
  mutate(drop_test = factor("a", levels = c("a", "b")))

dslite.server <- newDSLiteServer(
  tables = list(
    mtcars = mtcars,
    mtcars_group = mtcars_group
  )
)

dslite.server$config(defaultDSConfiguration(include=c("dsBase", "dsTidyverse")))
dslite.server$assignMethod("selectDS", "selectDS")
dslite.server$assignMethod("renameDS", "renameDS")
dslite.server$assignMethod("mutateDS", "mutateDS")
dslite.server$assignMethod("ifElseDS", "ifElseDS")
dslite.server$assignMethod("caseWhenDS", "caseWhenDS")
dslite.server$assignMethod("bindRowsDS", "bindRowsDS")
dslite.server$assignMethod("bindColsDS", "bindColsDS")
dslite.server$assignMethod("filterDS", "filterDS")
dslite.server$assignMethod("sliceDS", "sliceDS")
dslite.server$assignMethod("arrangeDS", "arrangeDS")
dslite.server$assignMethod("distinctDS", "distinctDS")
dslite.server$assignMethod("groupByDS", "groupByDS")
dslite.server$assignMethod("ungroupDS", "ungroupDS")
dslite.server$assignMethod("asTibbleDS", "asTibbleDS")

dslite.server$aggregateMethod("groupKeysDS", "groupKeysDS")

builder <- DSI::newDSLoginBuilder()

builder$append(
  server="server_1",
  url="dslite.server",
  table = "mtcars",
  driver = "DSLiteDriver")

logindata <- builder$build()
conns <- DSI::datashield.login(logins = logindata, assign = TRUE)

datashield.assign.table(
  conns = conns,
  table = "mtcars",
  symbol = "mtcars")

datashield.assign.table(
  conns = conns,
  table = "mtcars_group",
  symbol = "mtcars_group")
```

## Functions
### Select
`ds.select`, implements all the flexibility of `dplyr::select` in sub-setting columns. This includes:
(i) the ability to pass unquoted column names which are evaluated as columns of `df.name`, and
(ii) the ability to use tidyselect helper functions, such as `contains`. Below are some examples,
for full details see the documentation for `dplyr::select`.

#### Subset columns by name

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"
ds.select(df.name = "mtcars", tidy_expr = list(mpg, disp, wt), newobj = "new_df")
#> Error in .format_args_as_string(enquo(tidy_expr)): could not find function ".format_args_as_string"
ds.colnames("new_df")
#> Error:
#> ! There are some DataSHIELD errors, list them with datashield.errors()
```
#### Specify columns to exclude

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"
ds.select(df.name = "mtcars", tidy_expr = list(!mpg), newobj = "new_df")
#> Error in .format_args_as_string(enquo(tidy_expr)): could not find function ".format_args_as_string"
ds.colnames("new_df")
#> Error:
#> ! There are some DataSHIELD errors, list them with datashield.errors()
```
#### Specify a range of columns to keep

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"
ds.select(df.name = "mtcars", tidy_expr = list(mpg:drat), newobj = "new_df")
#> Error in .format_args_as_string(enquo(tidy_expr)): could not find function ".format_args_as_string"
ds.colnames("new_df")
#> Error:
#> ! There are some DataSHIELD errors, list them with datashield.errors()
```
#### Combine multiple conditions with '&'

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"
ds.select(df.name = "mtcars", tidy_expr = list(starts_with('c') & ends_with('b')), newobj = "new_df")
#> Error in .format_args_as_string(enquo(tidy_expr)): could not find function ".format_args_as_string"
ds.colnames("new_df")
#> Error:
#> ! There are some DataSHIELD errors, list them with datashield.errors()
```
#### Combine multiple conditions with '|'

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"
ds.select(df.name = "mtcars", tidy_expr = list(starts_with('c') | ends_with('b')), newobj = "new_df")
#> Error in .format_args_as_string(enquo(tidy_expr)): could not find function ".format_args_as_string"
ds.colnames("new_df")
#> Error:
#> ! There are some DataSHIELD errors, list them with datashield.errors()
```
### Rename
`ds.rename` renames columns within a server-side dataframe. Column names are passed unquoted and are
evaluated as column names within `df.name`. Below are some examples, see `dplyr::rename` for full details.


``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"
ds.rename(df.name = "mtcars", tidy_expr = list(effiency = mpg, power = hp), newobj = "new_df")
ds.colnames("new_df")
#> $server_1
#>  [1] "effiency" "cyl"      "disp"     "power"    "drat"     "wt"       "qsec"     "vs"       "am"      
#> [10] "gear"     "carb"
```

### Mutate
`ds.mutate` creates new columns in a server-side dataframe. These new columns are normally transformations of existing columns. This reduces the number of steps currently required in DataSHIELD, i.e. creating
a new vector and joining it back to an existing data frame. Again, column names are passed unquoted.
Below are some examples, see `dplyr::mutate` for full details.

#### Create columns which are transformations of existing variables:

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"
ds.mutate(
  df.name = "mtcars",
  tidy_expr = list(mpg_trans = cyl*1000, new_var = (hp-drat)/qsec),
  newobj = "new_df")
ds.colnames("new_df")
#> $server_1
#>  [1] "mpg"       "cyl"       "disp"      "hp"        "drat"      "wt"        "qsec"      "vs"       
#>  [9] "am"        "gear"      "carb"      "mpg_trans" "new_var"
ds.mean("mtcars$cyl")$Mean.by.Study
#>          EstimatedMean Nmissing Nvalid Ntotal
#> server_1        6.1875        0     32     32
ds.mean("new_df$mpg_trans")$Mean.by.Study
#>          EstimatedMean Nmissing Nvalid Ntotal
#> server_1        6187.5        0     32     32
ds.mean("mtcars$hp")$Mean.by.Study
#>          EstimatedMean Nmissing Nvalid Ntotal
#> server_1      146.6875        0     32     32
ds.mean("mtcars$drat")$Mean.by.Study
#>          EstimatedMean Nmissing Nvalid Ntotal
#> server_1      3.596563        0     32     32
ds.mean("mtcars$qsec")$Mean.by.Study
#>          EstimatedMean Nmissing Nvalid Ntotal
#> server_1      17.84875        0     32     32
ds.mean("new_df$new_var")$Mean.by.Study
#>          EstimatedMean Nmissing Nvalid Ntotal
#> server_1      8.372669        0     32     32
```

#### Choose where the new columns are positioned in the data frame with `.before` and `.after`:

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"
ds.mutate(
  df.name = "mtcars",
  tidy_expr = list(mpg_trans = cyl*1000, new_var = (hp-drat)/qsec),
  newobj = "new_df",
  .before = "disp")
ds.colnames("new_df")
#> $server_1
#>  [1] "mpg"       "cyl"       "mpg_trans" "new_var"   "disp"      "hp"        "drat"      "wt"       
#>  [9] "qsec"      "vs"        "am"        "gear"      "carb"
```

#### Only keep the newly created variables with `.keep`:

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"
ds.mutate(
  df.name = "mtcars",
  tidy_expr = list(mpg_trans = cyl*1000, new_var = (hp-drat)/qsec),
  newobj = "new_df",
  .keep = "none")
ds.colnames("new_df")
#> $server_1
#> [1] "mpg_trans" "new_var"
```

### if_else
A shinier version of `base::ifelse`. As with the other implementations of tidyverse, variable/object
names can be passed unquoted, this time in the `condition` argument.


``` r
ds.if_else(
  condition = list(mtcars$mpg > 20),
  "high",
  "low",
  newobj = "mpg_cat")

ds.table("mpg_cat")$output.list$TABLE_rvar.by.study_counts
#> 
#>  Data in all studies were valid 
#> 
#> Study 1 :  No errors reported from this study
#>        study
#> mpg_cat server_1
#>    high       14
#>    low        18
#>    NA          0
```

### bind_rows
Bind any number of data frames by row, making a longer result. This is similar to
do.call(rbind, dfs), but the output will contain all columns that appear in any of the inputs.


``` r
ds.dim("mtcars")[[1]]
#> [1] 32 11

ds.bind_rows(
  to_combine = list(mtcars, mtcars),
  newobj = "df_bound",
  datasources = conns
)

ds.dim("df_bound")[[1]]
#> [1] 64 11
```
The argument `.id` can be used to create an additional column which records which dataframe each
row came from:


``` r
ds.bind_rows(
  to_combine = list(mtcars, mtcars),
  newobj = "df_bound",
  datasources = conns,
  .id = "where_it_came_from"
)

ds.colnames("df_bound")
#> $server_1
#>  [1] "where_it_came_from" "mpg"                "cyl"                "disp"              
#>  [5] "hp"                 "drat"               "wt"                 "qsec"              
#>  [9] "vs"                 "am"                 "gear"               "carb"
```

### bind_cols
Bind any number of data frames by column, making a wider result. This is similar to do.call(cbind, dfs).


``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"

ds.bind_cols(
  to_combine = list(mtcars, mtcars),
  newobj = "df_bound",
  datasources = conns
)

ds.colnames("df_bound")
#> $server_1
#>  [1] "mpg...1"   "cyl...2"   "disp...3"  "hp...4"    "drat...5"  "wt...6"    "qsec...7"  "vs...8"   
#>  [9] "am...9"    "gear...10" "carb...11" "mpg...12"  "cyl...13"  "disp...14" "hp...15"   "drat...16"
#> [17] "wt...17"   "qsec...18" "vs...19"   "am...20"   "gear...21" "carb...22"
```
The argument `.name_repair` handles duplicate or broken names, e.g.

``` r
ds.colnames("mtcars")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"

ds.bind_cols(
  to_combine = list(mtcars, mtcars),
  newobj = "df_bound",
  .name_repair = "minimal",
  datasources = conns
)

ds.colnames("df_bound")
#> $server_1
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb" "mpg"  "cyl"  "disp"
#> [15] "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear" "carb"
```
### case_when
An extension of `dplyr::if_else` which allows the specification of multiple conditions. Extremely
useful for recoding variables.


``` r
  ds.case_when(
    tidy_expr = list(
      mtcars$mpg < 20 ~ "low",
      mtcars$mpg >= 20 & mtcars$mpg < 30 ~ "medium",
      mtcars$mpg >= 30 ~ "high"),
    newobj = "recoded",
    datasources = conns)

ds.table("recoded")$output.list$TABLE_rvar.by.study_counts
#> 
#>  Data in all studies were valid 
#> 
#> Study 1 :  No errors reported from this study
#>         study
#> recoded  server_1
#>   high          4
#>   low          18
#>   medium       10
#>   NA            0
```

You can also use the `.default` argument to control what happens if the condition is not met:


``` r
  ds.case_when(
    tidy_expr = list(
      mtcars$mpg < 20 ~ "low",
      mtcars$mpg >= 30 ~ "high"),
    newobj = "recoded_missing",
    .default = "something_missing",
    datasources = conns)

ds.table("recoded_missing")$output.list$TABLE_rvar.by.study_counts
#> 
#>  Data in all studies were valid 
#> 
#> Study 1 :  No errors reported from this study
#>                    study
#> recoded_missing     server_1
#>   high                     4
#>   low                     18
#>   something_missing       10
#>   NA                       0
```
See the help file for `dplyr::case_when` for all available arguments.

### filter
Subset set a data frame flexibly, retaining all rows that satisfy the condition supplied to
`expression`


``` r
ds.dim("mtcars")[[1]]
#> [1] 32 11
  ds.filter(
    df.name = "mtcars",
    tidy_expr = list(cyl == 4 & mpg > 20),
    newobj = "filtered",
    datasources = conns)
ds.dim("filtered")[[1]]
#> [1] 11 11
```

Use the .by argument to perform the filtering by a specified group.

``` r
ds.dim("mtcars")[[1]]
#> [1] 32 11
  ds.filter(
    df.name = "mtcars",
    tidy_expr = list(mpg > median(mpg)),
    .by = "cyl",
    newobj = "filtered_by",
    datasources = conns)
ds.dim("filtered_by")[[1]]
#> [1] 14 11
```

If filtering on an already grouped tibble, use the `preserve` argument to specify whether to retain
the original groups or recalculate groups based on the resulting data. See the help file for
`dplyr::filter` for more information.


``` r
ds.dim("mtcars")[[1]]
#> [1] 32 11
    ds.filter(
      df.name = "mtcars",
      tidy_expr = list(mpg > median(mpg)),
      .preserve = T,
      newobj = "preserved_t",
      datasources = conns
      )
ds.dim("preserved_t")[[1]]
#> [1] 15 11
```

### Slice
Subset rows using their positions. This is particularly useful if you want to take one observation
within a group (for example if you have repeated measures data, and want to take one measurements
per individual within an age group). Currently this can be done using `dh.createSubset`, however
this should be replaced by `group_by` and `slice` as it will be vastly quicker.


``` r
ds.dim("mtcars")[1]
#> $`dimensions of mtcars in server_1`
#> [1] 32 11
ds.slice(
  df.name = "mtcars",
  tidy_expr = list(1:10),
  newobj = "sliced_df"
)
ds.dim("sliced_df")[1]
#> $`dimensions of sliced_df in server_1`
#> [1] 10 11
```


``` r
ds.dim("mtcars")[1]
#> $`dimensions of mtcars in server_1`
#> [1] 32 11
ds.slice(
  df.name = "mtcars",
  tidy_expr = list(1),
  .by = "cyl",
  newobj = "sliced_df_group"
)
ds.dim("sliced_df_group")[1]
#> $`dimensions of sliced_df_group in server_1`
#> [1]  3 11
```

### Arrange
`arrange` is a more flexible version of `base::sort`, which orders the rows of a data frame by the values of selected columns.


``` r
ds.arrange(
  df.name = "mtcars",
  tidy_expr = list(mpg, cyl),
  newobj = "arranged_df",
  datasources = conns
  )
```
`arrange` can be used on a sorted data frame or tibble. The arrange argument `.by_group` (TRUE or FALSE) determines whether or not data is sorted by group.

### Grouping
Many data operations are performed on groups defined by variables. `group_by` takes an existing tibble or data frame and converts it into a grouped tibble. Subsequent operations can then be performed 'by group' rather than to the whole dataframe.

To group a data frame:

``` r
ds.group_by(
  df.name = "mtcars",
  tidy_expr = list(mpg, cyl),
  newobj = "grouped"
)
ds.class("grouped")[[1]]
#> [1] "grouped_df" "tbl_df"     "tbl"        "data.frame"
```

To ungroup a data frame:

``` r
ds.ungroup("grouped", "no_longer_grouped")
ds.class("no_longer_grouped")[[1]]
#> [1] "tbl_df"     "tbl"        "data.frame"
```

To return to clientside details of the groups, use `ds.group_keys`. Note that
this is subject to disclosure controls and will return an error if the number
of groups is too high.


``` r
my_groups <- ds.group_keys("mtcars_group")
my_groups
#> $server_1
#> # A tibble: 3 × 1
#>     cyl
#>   <dbl>
#> 1     4
#> 2     6
#> 3     8
```

### distinct
Use `distinct` to keep only unique rows. Leave the `expr` argument empty to check uniqueness across all variables:


``` r
ds.distinct(
  df.name = "mtcars",
  newobj = "distinct_df"
)
ds.dim("distinct_df")[[1]]
#> [1] 32 11
```
In this example the dimensions of the resulting dataset are the same because all rows are distinct. Alternatively you can specify a subset of variables in which to check for unique rows:


``` r
ds.distinct(
  df.name = "mtcars",
  tidy_expr = list(cyl, drat),
  newobj = "distinct_subset"
)
ds.dim("distinct_subset")[[1]]
#> [1] 26  2
ds.colnames("distinct_subset")[[1]]
#> [1] "cyl"  "drat"
```
See the help file of `dplyr::distinct` for information about other arguments.


### Convert objects to tibbles
'Tibbles' are the tidyverse version of dataframes. Currently within DataSHIELD it is possible
to convert dataframes and matrices to tibbles:


``` r
ds.class("mtcars")[[1]]
#> [1] "data.frame"
ds.as_tibble(
  x = "mtcars",
  newobj = "mtcars_tib",
  datasources = conns)
  ds.class("mtcars_tib")[[1]]
#> [1] "tbl_df"     "tbl"        "data.frame"
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.group_by.R
\name{ds.group_by}
\alias{ds.group_by}
\title{Group by one or more variables}
\usage{
ds.group_by(
  df.name = NULL,
  tidy_expr,
  .add = FALSE,
  .drop = TRUE,
  newobj = NULL,
  datasources = NULL
)
}
\arguments{
\item{df.name}{Character specifying a serverside data frame or tibble.}

\item{tidy_expr}{List of variables or computations to group by.}

\item{.add}{When FALSE, the default, \code{group_by()} will override existing groups. To add to the
existing groups, use .add = TRUE.}

\item{.drop}{Drop groups formed by factor levels that don't appear in the data? The default is
TRUE except when .data has been previously grouped with .drop = FALSE.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{DataSHIELD connections object.}
}
\value{
No return value, called for its side effects. A grouped data frame with class grouped_df
\code{newobj} is created on the server, unless the combination of \code{tidy_expr} and \code{.add}
yields a empty set of grouping columns, in which case a tibble will be created on the server.
}
\description{
DataSHIELD implentation of \code{dplyr::group_by}.
}
\examples{
\dontrun{
ds.group_by(
  df.name = "mtcars",
  expr = list(mpg, cyl),
  newobj = "grouped_df"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.as_tibble.R
\name{ds.as_tibble}
\alias{ds.as_tibble}
\title{Coerce a data frame or matrix to a tibble.}
\usage{
ds.as_tibble(
  x = NULL,
  .rows = NULL,
  .name_repair = "check_unique",
  rownames = NULL,
  newobj = NULL,
  datasources = NULL
)
}
\arguments{
\item{x}{A data frame or matrix.}

\item{.rows}{The number of rows, useful to create a 0-column tibble or just as an additional
check.}

\item{.name_repair}{Treatment of problematic column names:
\itemize{
  \item "minimal": No name repair or checks, beyond basic existence.
  \item "unique": Make sure names are unique and not empty.
  \item "check_unique": (default value), no name repair, but check they are unique.
  \item "universal": Make the names unique and syntactic.
}}

\item{rownames}{How to treat existing row names of a data frame or matrix:
\itemize{
  \item `NULL`: remove row names. This is the default.
  \item `NA`: keep row names.
  \item A string: the name of a new column. Existing rownames are transferred
  into this column and the \code{row.names} attribute is deleted.
  No name repair is applied to the new column name, even if `x` already contains
  a column of that name.
}}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{DataSHIELD connections object.}
}
\value{
No return value, called for its side effects. A tibble with the name specified by \code{newobj} is created on the server.
}
\description{
DataSHIELD implementation of \code{tibble::as_tibble}. Currently only implemented
for data frames and tibbles.
}
\examples{
\dontrun{
ds.as_tibble(
  x = "mtcars",
  newobj = "mtcars_tib",
  datasources = conns
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.group_keys.R
\name{ds.group_keys}
\alias{ds.group_keys}
\title{Describe the groups within a grouped tibble or data frame}
\usage{
ds.group_keys(df.name = NULL, datasources = NULL)
}
\arguments{
\item{df.name}{Character specifying a server-side tibble.}

\item{datasources}{DataSHIELD connections object.}
}
\value{
A data frame describing the groups.
}
\description{
DataSHIELD implementation of \code{dplyr::group_keys}.
}
\examples{
\dontrun{
my_groups <- ds.group_keys("grouped_df")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.select.R
\name{ds.select}
\alias{ds.select}
\title{Keep or drop columns using their names and types}
\usage{
ds.select(df.name = NULL, tidy_expr = NULL, newobj = NULL, datasources = NULL)
}
\arguments{
\item{df.name}{Character specifying a serverside data frame or tibble.}

\item{tidy_expr}{List of one or more unquoted expressions separated by commas. Variable names can
be used as if they were positions in the data frame, so expressions like x:y can be used to
select a range of variables.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{DataSHIELD connections object.}
}
\value{
No return value, called for its side effects. An object (typically a data frame or tibble)
with the name specified by \code{newobj} is created on the server.
}
\description{
DataSHIELD implentation of \code{dplyr::select}.
}
\examples{
\dontrun{
ds.select(
  df.name = "mtcars",
  tidy_expr = list(mpg, starts_with("t")),
  newobj = "df_subset",
  dataources = conns
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.distinct.R
\name{ds.distinct}
\alias{ds.distinct}
\title{Keep distinct/unique rows}
\usage{
ds.distinct(
  df.name = NULL,
  tidy_expr = NULL,
  .keep_all = FALSE,
  newobj = NULL,
  datasources = NULL
)
}
\arguments{
\item{df.name}{Character specifying a server-side data frame or tibble.}

\item{tidy_expr}{Optionally, list of variables to use when determining uniqueness. If there are multiple rows
for a given combination of inputs, only the first row will be preserved. If omitted, will use all
variables in the data frame.}

\item{.keep_all}{If TRUE, keep all variables in .data. If a combination of \code{expr} is not
distinct, this keeps the first row of values.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{DataSHIELD connections object.}
}
\value{
No return value, called for its side effects. An object (typically a data frame or tibble)
with the name specified by \code{newobj} is created on the server.
}
\description{
DataSHIELD implementation of \code{dplyr::distinct}.
}
\examples{
\dontrun{
ds.distinct(
  df.name = "mtcars",
  expr = list(mpg, cyl),
  newobj = "distinct_df"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.case_when.R
\name{ds.case_when}
\alias{ds.case_when}
\title{A general vectorised if-else}
\usage{
ds.case_when(
  tidy_expr = NULL,
  .default = NULL,
  .ptype = NULL,
  .size = NULL,
  newobj = NULL,
  datasources = NULL
)
}
\arguments{
\item{tidy_expr}{A list containing a sequence of two-sided formulas:
\itemize{
  \item The left hand side (LHS) determines which values match this case.
  \item The right hand side (RHS) provides the replacement value.
  \item The LHS inputs must evaluate to logical vectors.
  \item The RHS inputs will be coerced to their common type.
  }
  All inputs will be recycled to their common size. We encourage all LHS inputs to be the same size.
  Recycling is mainly useful for RHS inputs, where you might supply a size 1 input that will be
  recycled to the size of the LHS inputs. NULL inputs are ignored.}

\item{.default}{The value used when all of the LHS inputs return either FALSE or NA.}

\item{.ptype}{An optional prototype declaring the desired output type. If supplied, this overrides the common type of true, false, and missing.}

\item{.size}{An optional size declaring the desired output size. If supplied, this overrides the size of condition.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{datashield connections object.}
}
\value{
No return value, called for its side effects. A vector with the same size as the common
size computed from the inputs in \code{tidy_expr} and the same type as the common type of the
RHS inputs in \code{tidy_expr} is created on the server.
}
\description{
DataSHIELD implementation of \code{dplyr::case_when}.
}
\examples{
\dontrun{
## First log in to a DataSHIELD session with mtcars dataset loaded.

ds.case_when(
  tidy_expr = list(
    mtcars$mpg < 10 ~ "low",
    mtcars$mpg >= 10 & mtcars$mpg < 20 ~ "medium",
    mtcars$mpg >= 20 ~ "high"
  ),
  newobj = "test",
  datasources = conns
)

## Refer to the package vignette for more examples.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.arrange.R
\name{ds.arrange}
\alias{ds.arrange}
\title{Order the rows of a data frame by the values of selected columns}
\usage{
ds.arrange(
  df.name = NULL,
  tidy_expr = NULL,
  .by_group = NULL,
  newobj = NULL,
  datasources = NULL
)
}
\arguments{
\item{df.name}{Character specifying a server-side data frame or tibble.}

\item{tidy_expr}{A list containing variables, or functions of variables. Use \code{dplyr::desc()} to sort a
variable in descending order.}

\item{.by_group}{If TRUE, will sort first by grouping variable. Applies to grouped data frames
only.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{DataSHIELD connections object.}
}
\value{
No return value, called for its side effects. An object (typically a data frame or tibble) with the name specified by \code{newobj} is created on the server.
}
\description{
DataSHIELD implementation of \code{dplyr::arrange}.
}
\examples{
\dontrun{
ds.arrange(
  df.name = "mtcars",
  tidy_expr = list(drat),
  newobj = "sorted_df",
  datasources = conns
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.bind_rows.R
\name{ds.bind_rows}
\alias{ds.bind_rows}
\title{Bind multiple data frames by row.}
\usage{
ds.bind_rows(to_combine = NULL, .id = NULL, newobj = NULL, datasources = NULL)
}
\arguments{
\item{to_combine}{Data frames to combine. Each argument can either be a data frame, a list that
could be a data frame, or a list of data frames. Columns are matched by name, and any missing
columns will be filled with NA.}

\item{.id}{The name of an optional identifier column. Provide a string to create an output column
that identifies each input. The column will use names if available, otherwise it will use
positions.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{datashield connections object.}
}
\value{
No return value, called for its side effects. A data frame with the name specified by
\code{newobj} and the same type as the first element of \code{to_combine} is created on the
server.
}
\description{
DataSHIELD implementation of \code{dplyr::bind_rows}.
}
\examples{
\dontrun{
## First log in to a DataSHIELD session with mtcars dataset loaded.

ds.bind_rows(
  to_combine = list(mtcars, mtcars),
  newobj = "test",
  datasources = conns
)

## Refer to the package vignette for more examples.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.group_by.R
\name{ds.ungroup}
\alias{ds.ungroup}
\title{Remove grouping from a tibble or data frame}
\usage{
ds.ungroup(x = NULL, newobj = NULL, datasources = NULL)
}
\arguments{
\item{x}{a tibble or data frame.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{DataSHIELD connections object.}
}
\value{
No return value, called for its side effects. An ungrouped data frame or tibble is
created on the server.
}
\description{
DataSHIELD implementation of \code{dplyr::ungroup}.
}
\examples{
\dontrun{
ds.ungroup("grouped_df")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.slice.R
\name{ds.slice}
\alias{ds.slice}
\title{Subset rows using their positions}
\usage{
ds.slice(
  df.name = NULL,
  tidy_expr = NULL,
  .by = NULL,
  .preserve = FALSE,
  newobj = NULL,
  datasources = NULL
)
}
\arguments{
\item{df.name}{Character specifying a serverside data frame or tibble.}

\item{tidy_expr}{List, provide either positive values to keep, or negative values to drop. The values
provided must be either all positive or all negative. Indices beyond the number of rows in the
input are silently ignored.}

\item{.by}{Optionally, a selection of columns to group by for just this operation, functioning as
an alternative to \code{dplyr::group_by}}

\item{.preserve}{Relevant when the .data input is grouped. If .preserve = FALSE (the default),
the grouping structure is recalculated based on the resulting data, otherwise the grouping is
kept as is.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{DataSHIELD connections object.}
}
\value{
No return value, called for its side effects. An object (typically a data frame or tibble)
with the name specified by \code{newobj} is created on the server.
}
\description{
DataSHIELD implentation of \code{dplyr::slice}.
}
\examples{
\dontrun{
ds.slice(
  df.name = "mtcars",
  expr = list(1:10),
  .by = "cyl",
  newobj = "sliced_df"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.filter.R
\name{ds.filter}
\alias{ds.filter}
\title{Keep rows that match a condition}
\usage{
ds.filter(
  df.name = NULL,
  tidy_expr = NULL,
  .by = NULL,
  .preserve = FALSE,
  newobj = NULL,
  datasources = NULL
)
}
\arguments{
\item{df.name}{Character specifying a server-side data frame or tibble.}

\item{tidy_expr}{List of expressions that return a logical value, and are defined in terms of the
variables in \code{df.name}.}

\item{.by}{Optionally, a selection of columns to group by for just this operation, functioning as an alternative to \code{dplyr::group_by}}

\item{.preserve}{Relevant when the .data input is grouped. If .preserve = FALSE (the default),
the grouping structure is recalculated based on the resulting data, otherwise the grouping is
kept as is.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{DataSHIELD connections object.}
}
\value{
No return value, called for its side effects. An object (typically a data frame or tibble)
with the name specified by \code{newobj} is created on the server.
}
\description{
DataSHIELD implementation of \code{dplyr::filter}.
}
\examples{
\dontrun{
ds.filter(
  df.name = "mtcars",
  tidy_expr = list(cyl == 4 & mpg > 20),
  newobj = "filtered",
  datasources = conns
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.mutate.R
\name{ds.mutate}
\alias{ds.mutate}
\title{Create, modify, and delete columns}
\usage{
ds.mutate(
  df.name = NULL,
  tidy_expr = NULL,
  newobj = NULL,
  .keep = "all",
  .before = NULL,
  .after = NULL,
  datasources = NULL
)
}
\arguments{
\item{df.name}{Character specifying a server-side data frame or tibble.}

\item{tidy_expr}{List of tidyselect syntax to be passed to dplyr::mutate.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{.keep}{Control which columns from \code{df.name} are retained in the output. Options include:
\itemize{
  \item \code{"all"}: Retains all columns from \code{df.name}. This is the default.
  \item \code{"used"}: Retains only the columns used in \code{tidy_expr} to create new columns.
  \item \code{"unused"}: Retains only the columns not used in \code{tidy_expr} to create new columns. This is useful if you generate new columns but no longer need the columns used to generate them.
  \item \code{"none"}: Doesn't retain any extra columns from \code{df.name}. Only the grouping variables and columns created by \code{tidy_expr} are kept.
}
Grouping columns and columns created by \code{tidy_expr} are always kept.}

\item{.before}{<tidy-select> Optionally, control where new columns should appear (the default is
to add to the right hand side). See \code{tidy_expr} for more details.}

\item{.after}{<tidy-select> Optionally, control where new columns should appear (the default is
to add to the right hand side). See \code{tidy_expr} for more details.}

\item{datasources}{datashield connections object.}
}
\value{
No return value, called for its side effects. An object (typically a data frame or tibble)
with the name specified by \code{newobj} is created on the server.
}
\description{
DataSHIELD implementation of \code{dplyr::mutate}.
}
\examples{
\dontrun{
## First log in to a DataSHIELD session with mtcars dataset loaded.

ds.mutate(
  df.name = "mtcars",
  tidy_select = list(mpg_trans = cyl * 1000, new_var = (hp - drat) / qsec),
  newobj = "df_with_new_cols"
)

## Refer to the package vignette for more examples.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.rename.R
\name{ds.rename}
\alias{ds.rename}
\title{Rename columns}
\usage{
ds.rename(df.name = NULL, tidy_expr = NULL, newobj = NULL, datasources = NULL)
}
\arguments{
\item{df.name}{Character specifying a server-side data frame or tibble.}

\item{tidy_expr}{List with format new_name = old_name to rename selected variables.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{DataSHIELD connections object.}
}
\value{
No return value, called for its side effects. An object (typically a data frame or tibble)
with the name specified by \code{newobj} is created on the server.
}
\description{
DataSHIELD implementation of \code{dplyr::rename}.
}
\examples{
\dontrun{
## First log in to a DataSHIELD session with mtcars dataset loaded.

ds.rename(
  df.name = "mtcars",
  tidy_select = list(new_var_1 = mpg, new_var_2 = cyl),
  newobj = "df_renamed",
  dataources = conns
)

## Refer to the package vignette for more examples.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.bind_cols.R
\name{ds.bind_cols}
\alias{ds.bind_cols}
\title{Bind multiple data frames by column}
\usage{
ds.bind_cols(
  to_combine = NULL,
  .name_repair = c("unique", "universal", "check_unique", "minimal"),
  newobj = NULL,
  datasources = NULL
)
}
\arguments{
\item{to_combine}{Data frames to combine. Each argument can either be a data frame, a list that
could be a data frame, or a list of data frames. Columns are matched by name, and any missing
columns will be filled with NA.}

\item{.name_repair}{One of "unique", "universal", or "check_unique". See
\code{vctrs::vec_as_names()} for the meaning of these options.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{datashield connections object.}
}
\value{
No return value, called for its side effects. A data frame with the name specified by
\code{newobj} and the same type as the first element of \code{to_combine} is created on the
server.
}
\description{
DataSHIELD implementation of \code{dplyr::bind_cols}.
}
\examples{
\dontrun{
## First log in to a DataSHIELD session with mtcars dataset loaded.

ds.bind_cols(
  to_combine = list(mtcars, mtcars),
  .name_repair = "universal",
  newobj = "test",
  datasources = conns
)

## Refer to the package vignette for more examples.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds.if_else.R
\name{ds.if_else}
\alias{ds.if_else}
\title{Vectorised if-else}
\usage{
ds.if_else(
  condition = NULL,
  true = NULL,
  false = NULL,
  missing = NULL,
  ptype = NULL,
  size = NULL,
  newobj = NULL,
  datasources = NULL
)
}
\arguments{
\item{condition}{A list specifying a logical vector in tidyverse syntax, ie data and column names unquoted.}

\item{true}{Vector to use for TRUE value of condition.}

\item{false}{Vector to use for FALSE value of condition.}

\item{missing}{If not NULL, will be used as the value for NA values of condition. Follows the same size and type rules as true and false.}

\item{ptype}{An optional prototype declaring the desired output type. If supplied, this overrides the common type of true, false, and missing.}

\item{size}{An optional size declaring the desired output size. If supplied, this overrides the size of condition.}

\item{newobj}{Character specifying name for new server-side data frame.}

\item{datasources}{datashield connections object.}
}
\value{
No return value, called for its side effects. A vector with the same size as
\code{condition} and the same type as the common type of \code{true}, \code{false}, and
\code{missing} is created on the server.
}
\description{
DataSHIELD implementation of \code{dplyr::if_else}.
}
\examples{
\dontrun{
## First log in to a DataSHIELD session with mtcars dataset loaded.


## Refer to the package vignette for more examples.
}
}
